function [ newdata, error ] = dsetCorrBlk( data, loads )
%DSETCORRIOG uses the EEMlabs FDOMCORRECT012 function to correct the dataset In One Go
% This is the EEMlab version of the drEEM's FDOMCORRECT function
% FDOMCORRECT042 improves the spectral correction. It applies diferent
% corrections to each sample as it is described in the SampleLog file
% There are TWO input options with 6 or 12 input parameters.
%
% OPTION 1 (6 inputs):
% The simplest correction method requires 6 inputs and 2 outputs to produce spectrally corrected EEMs in RU units 
% There are no inner filter correction, blank subtraction or QS calibration steps:
% [Xc Arp]=fdomcorrect(X,Ex,Em,Emcor,Excor,W);   %spectral correction in RU units ALWAYS with landa = 350 nm
%
% OPTION 2 (12 inputs):
% FULL functionality
% [XcRU Arp IFCmat BcRU XcQS QS_RU]=fdomcorrect(X,Ex,Em,Emcor,Excor,W,RamOpt,A,B,T,Q,Qw);
%
% type HELP fdomcorrect for more information

%% === INIT
newdata = data;
error = {};

% %% === INFO
% label = 'Correct dataset';
% info = strcat({ ...
%     'Dataset corrected by EEMlab''s ''fdomcorrect012'' function', ...
%     'This is a revision of drEEM''s ''fdomcorrect'' v0.1.2', ...
%     '', ...
%     'Murphy KR, Stedmon CA, Graeber D, Bro R.', ...
%     '"Fluorescence spectroscopy and multi-way techniques. PARAFAC"' ...
%     'Analytical Methods 2013, 5, 6557-6566', ...
%     });
% uiwait(msgbox(info,label,'modal'));

%% === ERROR CHECKING
% here we define if use the 6-inputs (minimum) or the 12-inputs FDOMCORRECT function
% in some cases Em & Ex corrections are automatically done by the device

% check the AUTOMATIC GENERATION of Em & Ex corrections in the case of no
% having one of them and the fluorometer do the correction itself
contents = cellstr(loads.config.fluor.string);
device = contents{loads.config.fluor.value};


% if isempty(loads.Excor) || isempty(loads.Emcor)
%     switch device
%         case {'QuantaMaster', 'Aqualog'}
%             % QuantaMaster and Horiba Aqualog's EEMs EEMs are spectrally CORRECTED by default
%             % as Emcor & Excor are compulsory in 'fdomcorrect', we simulate correction
%             % scans with values set to 1 (NO correction). Then, both correction spectrums
%             % are manually generated by default in the range of 200-800 nm
%             % there is NO need to empair spectral correction data in this case
%             loads.Excor = [data.Ex ones(length(data.Ex),data.nSample)];
%             loads.Emcor = [data.Em ones(length(data.Em),data.nSample)];
%             info = strcat({ ...
%                 ['The EEMs have already been corrected by the ' device ' fluorimeter'], ...
%                 'A ''neutral'' correction file for the emission spectrum is generated', ...
%                 'A ''neutral'' correction file for the excitation spectrum is generated', ...
%                 });
%             label = 'Info';
%             icon = 'help';
%             uiwait(msgbox(info,label,icon,'modal'));
%             
%         otherwise
%             info = {'No corrections for Em or Ex spectrums', ...
%                 'The ''fdomcorrect'' function needs these corrections'};
%             label = 'Error';
%             icon = 'error';
%             uiwait(msgbox(info,label,icon,'modal'));
%             error = [ ...
%                 error, ...
%                 {'ERROR: EEMlab:dsetCorr:noCorrections For Spectrums'}, ...
%                 {'The ''fdomcorrect'' function needs these corrections'}, ...
%                 ];
%             return;
%     end
% end
if isempty(loads.Excor) || isempty(loads.Emcor)
    % QuantaMaster and Horiba Aqualog's EEMs EEMs are spectrally CORRECTED by default
    % as Emcor & Excor are compulsory in 'fdomcorrect', we simulate correction
    % scans with values set to 1 (NO correction). Then, both correction spectrums
    % are manually generated by default in the range of 200-800 nm
    % there is NO need to empair spectral correction data in this case
    loads.Excor = [data.Ex ones(length(data.Ex),data.nSample)];
    loads.Emcor = [data.Em ones(length(data.Em),data.nSample)];
    info = strcat({ ...
        'The user has NOT provided corrections for Em or Ex spectrums.', ...
        ['Then, it is supposed that the EEMs are automatically corrected by the ' device ' fluorimeter'], ...
        });
    label = 'Info';
    icon = 'help';
    uiwait(msgbox(info,label,icon,'modal'));
end

% BASIC FDOMCORRECT flag (TRUE if we have minimum inputs)
bflag = isempty(data.X) || isempty(data.Ex) || isempty(data.Em) || ...
    isempty(loads.Excor) || isempty(loads.Emcor);
if bflag
    info = {'Too few input parametres', ...
        'The ''fdomcorrect'' function needs at least 6 input valid parametres'};
    label = 'Error';
    icon = 'error';
    uiwait(msgbox(info,label,icon,'modal'));
    error = [ ...
        error, ...
        {'ERROR: EEMlab:dsetCorr:tooFewInputParametres'}, ...
        {'The ''fdomcorrect'' function needs at least 6 input parametres'}, ...
        ];
    return;
end

% so we have BASIC inputs, let's see what happens with water Raman scans
% WATER RAMAN scans flag (TRUE if we can obtain water Raman scans somehow)
wflag = isempty(loads.W) && isempty(loads.B);
if wflag
    info = {'Too few input parametres', ...
        'The ''fdomcorrect'' function needs water Raman scans (W) or dilution blanks (B)'};
    label = 'Error';
    icon = 'error';
    uiwait(msgbox(info,label,icon,'modal'));
    error = [ ...
        error, ...
        {'ERROR: EEMlab:dsetCorr:emptyInputParametres'}, ...
        {'The ''fdomcorrect'' function needs at least one from both water Raman scans (W) or blank EEMs (B)'}, ...
        ];
    return;    
end

% if we have BASIC inputs, let's see what happens with the more COMPLETE form...
% COMPLETE FDOMCORRECT flag to perform a better correction depending on the
% nuber of elective extra-arguments...
cflag = ~isempty(loads.A) || ~isempty(loads.B) || ...
    ~isempty(loads.Q) || ~isempty(loads.Qw);

%% == UPDATE DATASET

% ==== BASIC inputs
X = data;
Ex = data.Ex; % Excitation wavelengths
Em = data.Em; % Emission wavelengths
Emcor = loads.Emcor; % Emission spectral corrections for ALL EEMs and scans
Excor = loads.Excor; % Excitation spectral corrections for ALL EEMs and scans
contents = cellstr(loads.config.landa.string);
landa = str2double(contents{loads.config.landa.value});
if isempty(loads.W) % NO water Raman scans collected for the Dataset (DS)
    if landa ~= 350 % landa ~= 350 nm so we can NOT get water Raman scans from blanks!
        info = {'Dataset Raman scans MUST be extracted from the Milli-Q blanks at excitation wavelength 350 nm'};
        label = 'Error';
        icon = 'error';
        uiwait(msgbox(info,label,icon,'modal'));
        error = [ ...
            error, ...
            {'ERROR: EEMlab:dsetCorrSbsSpec:wrongExcitationWavelength'}, ...
            {'Excitation wavelegth for Raman scans MUST be 350 nm'}, ...
            {'Selected wavelength is 275 nm'}, ...
            ];
        return;
    else % landa == 350 so we extract water Raman scans from the Milli-Q blanks
        if isempty(loads.T) % NO loaded water Raman scans for the Milli-Q blanks
            info = {'ataset Raman scans can NOT be extracted from Milli-Q blanks'};
            label = 'Error';
            icon = 'error';
            uiwait(msgbox(info,label,icon,'modal'));
            error = [ ...
                error, ...
                {'ERROR: EEMlab:dsetCorrStepSpec:wrongExcitationWavelength'}, ...
                {'Dataset Raman scans can NOT be extracted from Milli-Q blanks'}, ...
                {'Load water Raman scans for the Milli-Q blanks'}, ...
                ];
            return;
        end
        W = loads.T;
    end
else % valid water Raman scans were specifically acquired for the dataset
    W = loads.W;
end
W = W(:,(W(1,:) >= min(Emcor(:,1))) & ... % CHECK spectral correction encompasses the full range of W
    (W(1,:) <= max(Emcor(:,1)))); % wavelength info in the first ROW

% ==== COMPLETE inputs
if cflag
    % Raman Options
    if landa == 350
        Em1=381; Em2=426;
        RamOpt = [landa Em1 Em2]; % default value also set in 'fdomcorrect'
    else % EEM dataset Raman options ONLY if landa <> 350 nm
        nsamp = size(W,1)-1;
        list = (1:nsamp)'; % type help RAMANINTEGRATIONRANGE for more information
        halfwidth = 1800; %maximum width (cm-1) of the scatter peak
        sequence = 6; %number of consecutive points identifying a sequence
        tolerance = 0; %minimum threshold identifying a non-zero slope
        plotview = 0;
        [~,IRmed,~] = ramanintegrationrange(W,list,landa,halfwidth,sequence,tolerance,plotview);
        % CHECK spectral correction encompasses the full range of IRmed
        IRmed(1) = max(IRmed(1),min(Emcor(:,1)));
        IRmed(2) = min(IRmed(2),max(Emcor(:,1)));
        RamOpt = [landa IRmed];
    end
    A = loads.A; % Absorbance scans
    B = loads.B; % Blanks
    T = loads.T; % Water Raman scans for blanks
    Q = loads.Q; % slopes for the QS dilution series
    Qw = loads.Qw; % Raman scans for QS dilution series dataset
    if ~isempty(Qw)
        Qw = Qw(:,(Qw(1,:) >= min(Emcor(:,1))) & ... % CHECK spectral correction encompasses the full range of Qw
            (Qw(1,:) <= max(Emcor(:,1)))); % wavelength info in the first ROW
    end
    dilfac = loads.dilfac; % Dilution factors
end

%% === DATA CORRECTION (fdomcorrect)
if cflag
    [XcRU] = fdomcorrect024_blk(X,Ex,Em,Emcor,Excor,W,RamOpt,A,B,T,Q,Qw);
else
    [XcRU]=fdomcorrect024_blk(X,Ex,Em,Emcor,Excor,W);
end
% undilute dataset
if ~isempty(loads.dilfac)
    XcRU = undilute(XcRU,dilfac);
end

%% === UPDATE DATASET
% 'AU' Arbitrary units
% 'QSE' Quinine Sulphate Equivalent units (for interlab comparison)
% 'RU275' Raman units (landa = 275 nm)
% 'RU350' Raman units (landa = 350 nm)
% 'RU' Raman units (landa = 350 nm)
% 'RFU' Relative Fluorescence units
newdata.X = XcRU;
if ~isempty(loads.Q) && ~isempty(loads.Qw)
    newdata.IntensityUnit = 'QSE';
else
    newdata.IntensityUnit = ['RU' num2str(landa)];
end

end
