function [ newloads, error ] = lodCorrDilBlanks( data, loads, samplelog, defaultT )
%LODCORRDILBLANKS preprocess EEM experiment subtracting blank to every sample
% if DEFAULTT (bool) the GUI generates (from the blanks themselves, at landa) the default water Raman scans for blanks

%% === INIT
newloads = loads;
error = {};
rflag = false; % recursive flag for 'getfilesinfolder'
pflag = false; % prepend flag for 'getfilesinfolder'
dflag = true; % deblank flag for 'targetsinlist'
home = pwd; % get current path

% %% === INFO
% label = 'Load of BLANK files';
% info = strcat({ ...
%     'Load of blanks to perform the blank subtraction on EEM dataset', ...
%     '', ...
%     'K.R.Murphy, K.D.Butler, R.G.M.Spencer, C.A.Stedmon, J.R.Boheme, G.R.Aiken', ...
%     '"Measurement of Dissolved Organic Matter Fluorescence in Aquatic Environments: An Interlaboratory Comparison"' ...
%     'Environ. Sci. Technol., 44 (2010), 9405-9412', ...
%     });
% uiwait(msgbox(info,label,'modal'));

%% === ERROR CHECKING
% checking for the correct configuration
if loads.config.B.value == 0 % No blank correction
    info = strcat({ ...
        'There are NO blank files for the blank subtraction', ...
        'Please, check the experiment configuration', ...
        'Blank files will NOT be loaded', ...
        });
    label = 'Error';
    icon = 'error';
    uiwait(msgbox(info,label,icon,'modal'));
    error = [ ...
        error, ...
        {'ERROR: EEMlab:lodCorrDilBlanks:unexistentCorrectionFiles'}, ...
        ];
    return;
end

% select samplelog tag and get the BLANK filelist from samplelog ('loglist')
headers = fieldnames(samplelog);
[tag,ok] = listdlg('Name','Select the metadata tag',...
    'PromptString','Select the tag name to load:',...
    'SelectionMode','single',...
    'ListSize',[160 250],...
    'ListString',headers);
if ~ok
    error = [error, {'ERROR: EEMlab:lodNormBlkRam:noSamplelogReference'}];
    return;
end
stag = headers{tag}; % selected tag
loglist = samplelog.(stag);

% select folder and get the BLANK filelist ('filelist')
title = 'Select the directory containing the set of BLANK files to load';
folder = uigetdir('',title); % get path to selected folder
if folder == 0 %user selected cancel
    error = [error, {'ERROR: EEMlab:lodCorrDilBlanks:userSelectedCancel'}];
    return;
end
filelist = getfilesinfolder(folder,rflag,pflag);

% match the selected BLANK dataset against the samplelog metadata list
[validFiles,outfle,~] = targetsinlist(loglist,filelist,dflag); % check if samplelog references are in filelist
if ~isempty(outfle) % NOT all the references in samplelog are IN the folder
    info = strcat({ ...
        'This folder does not contain the complete BLANK dataset referred to in the ''SampleLog'' metadata', ...
        '1.- complete the dataset and repeat the load OR...', ...
        '2.- delete unmatched records from ''SampleLog'' and repeat the metainformation & loads', ...
        ' ', ...
        'List of BLANK files NOT stored in folder:', ...
        outfle{:}, ...
        });
    label = 'Error';
    icon = 'error';
    uiwait(msgbox(info,label,icon,'modal'));
    error = [ ...
        error, ...
        {'ERROR: EEMlab:lodCorrIfeBlanks:incompleteDataset'}, ...
        {'List of BLANK files NOT stored in folder:'}, ...
        outfle{:}, ...
        ];
    return;
end
[~,outlog,~] = targetsinlist(filelist,loglist,dflag); % check if files in folder are referred to in samplelog
if ~isempty(outlog) % the folder contains extra files
    info = strcat({ ...
        'The folder contains BLANK files with no reference in the ''SampleLog'' metadata file', ...
        '1.- delete unmatched files from folder and repeat the load', ...
        ' ', ...
        'List of BLANK files NOT referenced in ''SampleLog'':', ...
        outlog{:}, ...
        });
    label = 'Error';
    icon = 'error';
    uiwait(msgbox(info,label,icon,'modal'));
    error = [ ...
        error, ...
        {'ERROR: EEMlab:lodCorrIfeBlanks:manyFilesInFolder'}, ...
        {'List of BLANK files NOT referenced in ''SampleLog'':'}, ...
        outlog{:}, ...
        ];
    return;
end

%% === FORMAT checking
[~,format] = strtok(validFiles{1},'.');
format = format(2:end);
contents = cellstr(loads.config.fluor.string);
device = contents{loads.config.fluor.value};
% DEVICES: QuantaMaster,SpexFluorog3,Aqualog,Fluoromax...
if (strcmpi(format,'txt') && strcmpi(device,'Horiba_QuantaMaster')) || ...
        (strcmpi(format,'csv') && strcmpi(device,'Horiba_SpexFluorolog3')) || ...
        (strcmpi(format,'csv') && strcmpi(device,'Horiba_Aqualog')) || ...
        (strcmpi(format,'csv') && strcmpi(device,'Horiba_FluoroMax')) || ...
        (strcmpi(format,'csv') && strcmpi(device,'Agilent_VarianCaryEclipse')) || ...
        (strcmpi(format,'txt') && strcmpi(device,'Jasco_FP8300'))
    lflag = device; % acquisition device flag for loads
else
    info = strcat({ ...
        ['The reference file format (' validFiles{1} ') do not matches with the format generated by the selected device'], ...
        'BLANK files will NOT be loaded', ...
        });
    label = 'Error';
    icon = 'error';
    uiwait(msgbox(info,label,icon,'modal'));
    error = [ ...
        error, ...
        {'ERROR: EEMlab:lodCorrDilBlanks:wrongFileFormat'}, ...
        ];
    return;    
end

%% === DATA LOAD
cd(folder);
switch lflag

    % QuantaMaster EEMs
    case 'Horiba_QuantaMaster'
        [X,Emmat,Exmat,filelist,err] = readineems4qms();
        if ~isempty(err)
            error = [error err{:}];
            label = 'Error';
            icon = 'error';
            uiwait(msgbox(err(:),label,icon,'modal'));            
            return;
        end

    % Other fluorometers directly load RAW EEMs dataset    
    case {'Horiba_SpexFluorolog3', 'Horiba_FluoroMax'}
        type = 1; % Ex in cols, Em in rows
        [range,~,err] = getrange(validFiles{1});
        headers = [1 1];
        display = 0;
        outdat = 0;
        if isempty(err)
            [X,Emmat,Exmat,filelist,~] = readineems(type,format,range,headers,display,outdat);
        else
            error = [error err{:}];
            label = 'Error';
            icon = 'error';
            uiwait(msgbox(err(:),label,icon,'modal'));            
            return;
        end

    % Horiba Aqualog EEMs
    case 'Horiba_Aqualog'
        % Ex in columns and Em in rows, Ex in descending order (AquaLog)
        type = 3;
        % Aqualog's valid numeric range is 'A3' --> [0 2] para CSVREAD
        % eliminamos filas 1 y 2 donde aparecen datos NO numricos 
        % en fila 2 aparecen Ex que nos cargamos --> headers = [0 1]
        % como en CSVREAD el rango se epecifica numricamente desde [0 0]
        % (zero-based) es decir 'A1' --> [0 0] por eso 'A3' --> [0 2]
        % ver tambin READINEEMS
        validRange = [0 2]; % 'A3' (las dos primeras filas son de valores NO numricos)
        [range,~,err] = getrange(validFiles{1}, validRange);
        % [0 1]:  present for Em but not Ex - this is usual for Varian and AquaLog files once text are excluded from range
        headers = [0 1];
        display = 0;
        outdat = 0;
        if isempty(err)
            [X,Emmat,Exmat,filelist,~] = readineems(type,format,range,headers,display,outdat);
        else
            error = [error err{:}];
            label = 'Error';
            icon = 'error';
            uiwait(msgbox(err(:),label,icon,'modal'));            
            return;
        end

    % Varian Cary Eclipse
    case 'Agilent_VarianCaryEclipse'
        % Ex in columns and Em in rows, with Em headers displayed in every second column (Varian)
        type = 2;
        validRange = [0 2]; % 'A3' (las dos primeras filas son de valores NO numricos)
        [range,~,err] = getrange(validFiles{1}, validRange);
        headers = [0 1]; % [0 1]:  present for Em but not Ex - this is usual for Varian and AquaLog files once text are excluded from range
        display = 0;
        outdat = 0;
        if isempty(err)
            [X,Emmat,Exmat,filelist,~] = readineems(type,format,range,headers,display,outdat);
        else
            error = [error err{:}];
            label = 'Error';
            icon = 'error';
            uiwait(msgbox(err(:),label,icon,'modal'));
            return;
        end
        
    % FP8300 (JASCO), UCA (Universidad de Cdiz) dataset (TXT files)
    case 'Jasco_FP8300'
        [X,Emmat,Exmat,filelist,err] = readineems4jsc();
        if ~isempty(err)
            error = [error err{:}];
            label = 'Error';
            icon = 'error';
            uiwait(msgbox(err(:),label,icon,'modal'));            
            return;
        end
        
    otherwise
        %other BLANK files for Hitachi, Jasco
        label = 'Load of BLANK correction files';
        info = strcat({ ...
            'The load of the BLANK correction files for the ' device ' is not implemented yet', ...
            'Blank files will NOT be loaded', ...
            });
        uiwait(msgbox(info,label,'modal'));
        error = [ ...
            error, ...
            {'ERROR: EEMlab:lodCorrDilBlanks:notImplementedOption'}, ...
            {'BLANK correction files are NOT loaded'}, ...
            ];
        return;
        
end
cd(home);

%% === UPDATE DATASET
disp('aligning dilution blanks');
B = alignds(samplelog,{data.header,data.filelist},{stag,filelist,X});
newloads.B = B;

if defaultT
    % DEFAULT T (water Raman scans at landa from blanks)
    % this is used as default value if we do NOT have water Raman scans from blanks
    % this value can be updated in menu LODNORMBLKRAM if we DO have water Raman scans from blanks
    % the same functionality is also added in FDOMCORRECT script
    contents = cellstr(loads.config.landa.string);
    landa = contents{loads.config.landa.value};
    disp(['aligning DEFAULT water Raman scans for blanks at ' landa ' nm']);
    Exmat_aligned = alignds(samplelog,{data.header,data.filelist},{stag,filelist,Exmat});
    nsamp = size(B,1);
    nem = size(B,2);
    T = zeros(nsamp,nem);
    for i = 1 : nsamp
        scan = B(i,:,Exmat_aligned(i,:) == str2double(landa));
        T(i,:) = scan;
    end
    T = [Emmat(:,1)'; T]; % 1st ROW is wavelengths
    newloads.T = T; % we need 'Exmat' for water Raman scans extraction if there is NO T in the experiment
end

end
