function [ newloads, error ] = lodCorrIfeAbs( data, loads, samplelog )
%LODCORRIFEABS loads absorbance scans for a further Inner Filter Effect
%(IFE) correction (Lakowicz method)

%% === INIT
newloads = loads;
error = {};
rflag = false; % recursive flag for 'getfilesinfolder'
pflag = false; % prepend flag for 'getfilesinfolder'
dflag = true; % deblank flag for 'targetsinlist'
home = pwd; % get current path

% %% === INFO
% label = 'Load of absorbance scans';
% info = strcat({ ...
%     'Inner Filter Effect (IFE) corrected with Lakowicz method', ...
%     '', ...
%     'J.R. Lakowicz, "Principles of Fluorescence Spectroscopy", 3rd edition, Springer, Baltimore, Maryland, 2006', ...
%     });
% uiwait(msgbox(info,label,'modal'));

%% === ERROR CHECKING
% checking for the correct configuration
if loads.config.A.value == 0 % Absorbance already corrected
    info = strcat({ ...
        'There are NO absorbance files for the IFE correction', ...
        'Please, check the experiment configuration', ...
        'Absorbance files will NOT be loaded', ...
        });
    label = 'Error';
    icon = 'error';
    uiwait(msgbox(info,label,icon,'modal'));
    error = [ ...
        error, ...
        {'ERROR: EEMlab:lodCorrIfeAbs:unexistentCorrectionFiles'}, ...
        ];
    return;
end

% select samplelog tag and get the ABSORBANCE filelist from samplelog ('loglist')
headers = fieldnames(samplelog);
[tag,ok] = listdlg('Name','Select the metadata tag',...
    'PromptString','Select the tag name to load:',...
    'SelectionMode','single',...
    'ListSize',[160 250],...
    'ListString',headers);
if ~ok
    error = [error, {'ERROR: EEMlab:lodCorrIfeAbs:noSamplelogReference'}];
    return;
end
stag = headers{tag}; % selected tag
loglist = samplelog.(stag);

% select folder and get the ABSORBANCE filelist ('filelist')
title = 'Select the directory containing the set of ABSORBANCE scans to load';
folder = uigetdir('',title); % get path to selected folder
if folder == 0 %user selected cancel
    error = [error, {'ERROR: EEMlab:lodCorrIfeAbs:userSelectedCancel'}];
    return;
end
filelist = getfilesinfolder(folder,rflag,pflag);

% match the selected CORRECTION dataset against the samplelog metadata list
[validFiles,outfle,~] = targetsinlist(loglist,filelist,dflag); % check if samplelog references are in filelist
if ~isempty(outfle) % NOT all the references in samplelog are IN the folder
    info = strcat({ ...
        'This folder does not contain the complete ABSORBANCE dataset referred to in the ''SampleLog'' metadata', ...
        '1.- complete the dataset and repeat the load OR...', ...
        '2.- delete unmatched records from ''SampleLog'' and repeat the metainformation & loads', ...
        ' ', ...
        'List of ABSORBANCE files NOT stored in folder:', ...
        outfle{:}, ...
        });
    label = 'Error';
    icon = 'error';
    uiwait(msgbox(info,label,icon,'modal'));
    error = [ ...
        error, ...
        {'ERROR: EEMlab:lodCorrIfeAbs:incompleteDataset'}, ...
        {'List of ABSORBANCE files NOT stored in folder:'}, ...
        outfle{:}, ...
        ];
    return;
end
[~,outlog,~] = targetsinlist(filelist,loglist,dflag); % check if files in folder are referred to in samplelog
if ~isempty(outlog) % the folder contains extra files
    info = strcat({ ...
        'The folder contains ABSORBANCE files with no reference in the ''SampleLog'' metadata file', ...
        '1.- delete unmatched files from folder and repeat the load', ...
        ' ', ...
        'List of ABSORBANCE files NOT referenced in ''SampleLog'':', ...
        outlog{:}, ...
        });
    label = 'Error';
    icon = 'error';
    uiwait(msgbox(info,label,icon,'modal'));
    error = [ ...
        error, ...
        {'ERROR: EEMlab:lodCorrIfeAbs:manyFilesInFolder'}, ...
        {'List of ABSORBANCE files NOT referenced in ''SampleLog'':'}, ...
        outlog{:}, ...
        ];
    return;
end

%% === FORMAT checking
[~,format] = strtok(validFiles{1},'.');
format = format(2:end);
contents = cellstr(loads.config.spect.string);
device = contents{loads.config.spect.value};
% DEVICES: 'Hitachi','VarianCaryEclipse','VarianCary50','Agilent 8453','Jenway_7315','PerkinElmer'
if (strcmpi(format,'csv') && strcmpi(device,'Hitachi')) || ...
        (strcmpi(format,'csv') && strcmpi(device,'Varian_Cary4E')) || ...
        (strcmpi(format,'csv') && strcmpi(device,'Varian_Cary50')) || ...
        (strcmpi(format,'csv') && strcmpi(device,'Agilent_8453')) || ...
        (strcmpi(format,'txt') && strcmpi(device,'Jenway_7315')) || ...
        (strcmpi(format,'csv') && strcmpi(device,'PerkinElmer'))
    lflag = device; % load flag
else
    info = strcat({ ...
        ['The reference file format (' validFiles{1} ') do not matches with the format generated by the selected device'], ...
        'Correction files will NOT be loaded', ...
        });
    label = 'Error';
    icon = 'error';
    uiwait(msgbox(info,label,icon,'modal'));
    error = [ ...
        error, ...
        {'ERROR: EEMlab:lodCorrIfeAbs:wrongFileFormat'}, ...
        ];
    return;    
end

%% === DATA LOAD
cd(folder);
switch lflag
    % Hitachi, EPSA-UPV dataset (CSV files)
    case 'Hitachi'
        [X,~,wave,filelist,err] = readinscans4hti();
        if ~isempty(err)
            error = [error err{:}];
            label = 'Error';
            icon = 'error';
            uiwait(msgbox(err(:),label,icon,'modal'));
            return;
        end
    % Other spectrophotometers as Cary 4E (Varian), drEEM dataset (CSV files)
    case {'Varian_Cary4E', 'Varian_Cary50', 'PerkinElmer'}
        type = 'Absorbance';
        [xls_range,~,err] = getrange(validFiles{1});
        display = 0;
        outdat = 0;
        if isempty(err)
            [X,~,wave,filelist] = readinscans(type,format,xls_range,display,outdat);
        else
            error = [error err{:}];
            label = 'Error';
            icon = 'error';
            uiwait(msgbox(err(:),label,icon,'modal'));
            return;
        end
    % Agilent 8453, A. HECHAVARRIA dataset (CSV files)
    case 'Agilent_8453'
        % AGILENT saves absorbance CSV files as UNICODE (UTF-16 strings)
        % but DLMREAD reads UTF-8 files. Then we have to open AGILENT
        % files and save them as UTF-8 formatted files!!!
        for i = 1 : length(validFiles)
            text = fileread(validFiles{i});
            ind = text == 0;
            text(ind) = ''; % we delete 0h characters to get the UTF8 format
            fid = fopen(validFiles{i},'w');
            fprintf(fid,'%s',text);
            fclose(fid);
        end
        validRange = [0 1]; % 'A2' (la primera fila contiene de valores NO numricos)
        [xls_range,~,err] = getrange(validFiles{1}, validRange);
        if isempty(err)
            display = 0;
            outdat = 0;
            type = 'Agilent 8453 absorbance scans';
            format = [format '_1_2']; % columna 1 longitudes de onda y columna 2 absorbancias
            [X,~,wave,filelist] = readinscans(type,format,xls_range,display,outdat);
        else
            error = [error err{:}];
            label = 'Error';
            icon = 'error';
            uiwait(msgbox(err(:),label,icon,'modal'));
            return;
        end
    % 7315 (Jenway), UCa (Universidad de Cdiz) dataset (TXT files)
    case 'Jenway_7315'
        [X,~,wave,filelist,err] = readinscans4jen();
        if ~isempty(err)
            error = [error err{:}];
            label = 'Error';
            icon = 'error';
            uiwait(msgbox(err(:),label,icon,'modal'));
            return;
        end
end
cd(home);

%% === UPDATE DATASET
A = alignds(samplelog,{data.header,data.filelist},{stag,filelist,X});
A = [wave; A];
newloads.A = A;

end