function [ newloads, error ] = lodCorrSpecEx( data, loads, samplelog )
%LODSPECEX loads excitation scans for spectral correction

%% === INIT
newloads = loads;
error = {};
rflag = false; % recursive flag for 'getfilesinfolder'
pflag = false; % prepend flag for 'getfilesinfolder'
dflag = true; % deblank flag for 'targetsinlist'
home = pwd; % get current path

%% === INFO

%% === ERROR CHECKING
% checking for the correct configuration

% select samplelog tag and get the EEM list of samplelog's files ('loglist')
headers = fieldnames(samplelog);
[tag,ok] = listdlg('Name','Select the metadata tag',...
    'PromptString','Select the tag name to load:',...
    'SelectionMode','single',...
    'ListSize',[160 250],...
    'ListString',headers);
if ~ok
    error = [error, {'ERROR: EEMlab:lodCorrSpecEx:noSamplelogReference'}];
    return;
end
stag = headers{tag}; % selected tag
loglist = samplelog.(stag);

% select folder and get the CORRECTION filelist ('filelist')
title = 'Select the directory containing the set of spectral CORRECTION files to load';
folder = uigetdir('',title); % get path to selected folder
if folder == 0 % user selected cancel
    error = [error, {'ERROR: EEMlab:lodCorrSpecEx:userSelectedCancel'}];
    return;
end
filelist = getfilesinfolder(folder,rflag,pflag);

% match the selected CORRECTION dataset against the samplelog metadata list
[validFiles,outfle,~] = targetsinlist(loglist,filelist,dflag); % check if samplelog references are in filelist
if ~isempty(outfle) % NOT all the references in samplelog are IN the folder
    info = strcat({ ...
        'This folder does not contain the complete CORRECTION dataset referred to in the ''SampleLog'' metadata', ...
        '1.- complete the dataset and repeat the load OR...', ...
        '2.- delete unmatched records from ''SampleLog'' and repeat the metainformation & loads', ...
        ' ', ...
        'List of CORRECTION files NOT stored in folder:', ...
        outfle{:}, ...
        });
    label = 'Error';
    icon = 'error';
    uiwait(msgbox(info,label,icon,'modal'));
    error = [ ...
        error, ...
        {'ERROR: EEMlab:lodCorrSpecEx:incompleteDataset'}, ...
        {'List of CORRECTION files NOT stored in folder:'}, ...
        outfle{:}, ...
        ];
    return;
end
[~,outlog,~] = targetsinlist(filelist,loglist,dflag); % check if files in folder are referred to in samplelog
if ~isempty(outlog) % the folder contains extra files
    info = strcat({ ...
        'The folder contains CORRECTION files with no reference in the ''SampleLog'' metadata file', ...
        '1.- delete unmatched files from folder and repeat the load', ...
        ' ', ...
        'List of CORRECTION files NOT referenced in ''SampleLog'':', ...
        outlog{:}, ...
        });
    label = 'Error';
    icon = 'error';
    uiwait(msgbox(info,label,icon,'modal'));
    error = [ ...
        error, ...
        {'ERROR: EEMlab:lodCorrSpecEx:manyFilesInFolder'}, ...
        {'List of CORRECTION files NOT referenced in ''SampleLog'':'}, ...
        outlog{:}, ...
        ];
    return;
end

%% === FORMAT checking
[~,format] = strtok(validFiles{1},'.');
format = format(2:end);
contents = cellstr(loads.config.fluor.string);
device = contents{loads.config.fluor.value};
% DEVICES: QuantaMaster,SpexFluorog3,Aqualog,Fluoromax, Jasco...
if (strcmpi(format,'txt') && strcmpi(device,'Horiba_QuantaMaster')) || ...
        (strcmpi(format,'csv') && strcmpi(device,'Horiba_SpexFluorolog3')) || ...
        (strcmpi(format,'csv') && strcmpi(device,'Horiba_Aqualog')) || ...
        (strcmpi(format,'csv') && strcmpi(device,'Horiba_FluoroMax')) || ...
        (strcmpi(format,'csv') && strcmpi(device,'Agilent_VarianCaryEclipse') && (newloads.config.Excor.value == 1)) || ...
        (strcmpi(format,'txt') && strcmpi(device,'Jasco_FP8300'))
    lflag = device; % acquisition device flag for loads
else
    info = strcat({ ...
        ['The reference file format (' validFiles{1} ') do not matches with the format generated by the selected device'], ...
        'EEM files will NOT be loaded', ...
        });
    label = 'Error';
    icon = 'error';
    uiwait(msgbox(info,label,icon,'modal'));
    error = [ ...
        error, ...
        {'ERROR: EEMlab:lodCorrSpecEx:wrongFileFormat'}, ...
        ];
    return;    
end

%% === DATA LOAD
cd(folder);
switch lflag
    
    case {'Horiba_QuantaMaster', 'Horiba_Aqualog'}
        % eemlab and gold datasets
        label = 'Load of EX spectral correction files';
        info = ['The ' device ' fluorometer automatically corrects spectrum for excitation wavelengths'];
        uiwait(msgbox(info,label,'modal'));
        return;

    case 'Horiba_SpexFluorolog3'
        % drEEM dataset
        [xls_range,~,err] = getrange(validFiles{1});
        if isempty(err)
            display = 0;
            outdat = 0;
            type = 'Horiba SpexFluorolog3 Excor';
            [X,~,wave,filelist] = readinscans(type,format,xls_range,display,outdat);
        else
            error = [error err{:}];
            return;
        end
        
    case 'Agilent_VarianCaryEclipse'
        % A. Hechavarria dataset --> VarianCary can correct automatically the EEMs
        % but it can also provide raw EEMs to correct them after (THIS case)
        % 'A3' (TWO header rows with NO numerical values) and finishes with
        % NO numerical metainformation added to the end of the Excor file.
        % Then, first we delete metainformation looking for an empty line
        % (this is the border between numerical info and mertainformation)
        % like '0D 0A 0D 0A' (0Dh -> 'Carriage return'; 0Ah -> 'New line')
        % and once found we truncate the file to avoid this NON numerical
        % metainformation
        pattern = [char(13) char(10) char(13) char(10)]; % '0D 0A 0D 0A'
        for i = 1 : length(validFiles)
            text = fileread(validFiles{i});
            ind = strfind(text,pattern);
            if ~isempty(ind)
                text(ind(1):end) = '';
                fid = fopen(validFiles{i},'w');
                fprintf(fid,'%s',text);
                fclose(fid);
            end
        end
        validRange = [0 2]; % from A3 to the end of the file
        [xls_range,~,err] = getrange(validFiles{1}, validRange);
        % finally we discard the third unuseful column, added by the VarianCary
        xls_range = strrep(xls_range,'C','B');
        if isempty(err)
            display = 0;
            outdat = 0;
            type = 'Agilent VarianCary Excor';
            [X,~,wave,filelist] = readinscans(type,format,xls_range,display,outdat);
        else
            error = [error err{:}];
            label = 'Error';
            icon = 'error';
            uiwait(msgbox(err(:),label,icon,'modal'));
            return;
        end
        
    otherwise
        %other CORRECTION files for Fluoromax, Hitachi, Jasco
        label = 'Load of EX spectral correction files';
        info = ['The load of the EX spectral correction files for the ' device ' is not implemented yet'];
        icon = 'warn';
        uiwait(msgbox(info,label,icon,'modal'));
        error = [ ...
            error, ...
            {'ERROR: EEMlab:lodCorrSpecEx:functionalityNotImplemented'}, ...
            {'EX spectral correction files are NOT loaded'}, ...
            ];
        return;
end
cd(home);

%% === UPDATE DATASET
% Emission spectral corrections for ALL EEMs and scans!
Excor = alignds(samplelog,{data.header,data.filelist},{stag,filelist,X});
Excor = [wave; Excor]'; % add wavelength information (first column) to 'fdomcorrect' inputs
newloads.Excor = Excor;

end