function [ newdata, newloads, error ] = lodDsetEems( data, loads, samplelog )
%LODDSETEEMS loads new samples from a set of files and generates a drEEM
%dataset

%% === INIT
newdata = data;
newloads = loads;
error = {};
rflag = false; % recursive flag for 'getfilesinfolder'
pflag = false; % prepend flag for 'getfilesinfolder'
dflag = true; % deblank flag for 'targetsinlist'

%% === INFO

%% === ERROR CHECKING
% checking for the correct configuration
if loads.config.eems.value == 0 % NO EEMs included in experiment!
    info = strcat({ ...
        'There are NO EEM files included in this experiment', ...
        'Please, check the experiment configuration', ...
        'EEM files will NOT be loaded', ...
        });
    label = 'Error';
    icon = 'error';
    uiwait(msgbox(info,label,icon,'modal'));
    error = [ ...
        error, ...
        {'ERROR: EEMlab:lodDsetEems:unexistentDatasetFiles'}, ...
        ];
    return;
end

% select samplelog tag and get the EEM list of samplelog's files ('loglist')
headers = fieldnames(samplelog);
[tag,ok] = listdlg('Name','Select the metadata tag',...
    'PromptString','Select the tag name to load:',...
    'SelectionMode','single',...
    'ListSize',[160 250],...
    'ListString',headers);
if ~ok
    error = [error, {'ERROR: EEMlab:lodDsetEems:userSelectedCancel'}];
    return;
end
stag = headers{tag}; % selected tag
loglist = samplelog.(stag);

% select folder and get the EEM filelist ('filelist')
home = pwd; % get current path
title = 'Select the directory containing the set of EEM files to load';
folder = uigetdir('',title); % get path to selected folder
if folder == 0
    error = [error, {'ERROR: EEMlab:lodDsetEems:userSelectedCancel'}];
    return;
end
filelist = getfilesinfolder(folder,rflag,pflag);

% match the selected EEM dataset against the samplelog metadata list
[validFiles,outfle,~] = targetsinlist(loglist,filelist,dflag); % check if samplelog references are in filelist
if ~isempty(outfle) % NOT all the references in samplelog are IN the folder
    info = strcat({ ...
        'The folder does not contain the complete EEM dataset referred to in the ''SampleLog'' metadata', ...
        '1.- complete the dataset and repeat the EEM load OR...', ...
        '2.- delete unmatched records from ''SampleLog'' and repeat the metainformation & EEM loads', ...
        ' ', ...
        'List of SAMPLELOG''s EEM files NOT stored in folder:', ...
        outfle{:}, ...
        });
    label = 'Error';
    icon = 'error';
    uiwait(msgbox(info,label,icon,'modal'));
    error = [ ...
        error, ...
        {'ERROR: EEMlab:lodDsetEems:incompleteDataset'}, ...
        {'List of SampleLog''s EEM files NOT stored in folder:'}, ...
        outfle{:}, ...
        ];
    return;
end
[~,outlog,~] = targetsinlist(filelist,loglist,dflag); % check if files in folder are referred to in samplelog
if ~isempty(outlog) % the folder contains extra files
    info = strcat({ ...
        'The folder contains EEM files with no reference in the ''SampleLog'' metadata file', ...
        '1.- delete unmatched files from folder and repeat the EEM load', ...
        ' ', ...
        'List of EEM files NOT referenced in ''SampleLog'':', ...
        outlog{:}, ...
        });
    label = 'Error';
    icon = 'error';
    uiwait(msgbox(info,label,icon,'modal'));
    error = [ ...
        error, ...
        {'ERROR: EEMlab:lodDsetEems:manyEemFilesInFolder'}, ...
        {'List of EEM files NOT referenced in ''SampleLog'':'}, ...
        outlog{:}, ...
        ];
    return;
end

%% === FORMAT checking
[~,format] = strtok(validFiles{1},'.');
format = format(2:end);
contents = cellstr(loads.config.fluor.string);
device = contents{loads.config.fluor.value};
% DEVICES: QuantaMaster,SpexFluorog3,Aqualog,Fluoromax, Jasco...
if (strcmpi(format,'csv') && strcmpi(device,'STANDARD')) || ...
        (strcmpi(format,'txt') && strcmpi(device,'Horiba_QuantaMaster')) || ...
        (strcmpi(format,'csv') && strcmpi(device,'Horiba_SpexFluorolog3')) || ...
        (strcmpi(format,'csv') && strcmpi(device,'Horiba_Aqualog')) || ...
        (strcmpi(format,'csv') && strcmpi(device,'Horiba_FluoroMax')) || ...
        (strcmpi(format,'csv') && strcmpi(device,'Agilent_VarianCaryEclipse')) || ...
        (strcmpi(format,'txt') && strcmpi(device,'Jasco_FP8300')) || ...
        (strcmpi(format,'xlsx') && strcmpi(device,'Hitachi_F2700'))
        lflag = device; % acquisition device flag for loads
else
    info = strcat({ ...
        ['The reference file format (' validFiles{1} ') do not matches with the format generated by the selected device'], ...
        'EEM files will NOT be loaded', ...
        });
    label = 'Error';
    icon = 'error';
    uiwait(msgbox(info,label,icon,'modal'));
    error = [ ...
        error, ...
        {'ERROR: EEMlab:lodDsetEems:wrongFileFormat'}, ...
        ];
    return;    
end
    
%% === DATA LOAD
cd(folder);
switch lflag

    %  QuantaMaster (HORIBA), EPSA-UPV dataset (TXT files)
    case 'Horiba_QuantaMaster'
        [X,Emmat,Exmat,filelist,err] = readineems4qms();
        if ~isempty(err)
            error = [error err{:}];
            label = 'Error';
            icon = 'error';
            uiwait(msgbox(err(:),label,icon,'modal'));            
            return;
        end

    % Other fluorometers directly load RAW EEMs dataset (drEEM dataset, STANDARD dataset)
    case {'STANDARD', 'Horiba_SpexFluorolog3', 'Horiba_FluoroMax'}
        type = 1; % Ex in cols, Em in rows
        [range,~,err] = getrange(validFiles{1});
        headers = [1 1];
        display = 0;
        outdat = 0;
        if isempty(err)
            [X,Emmat,Exmat,filelist,~] = readineems(type,format,range,headers,display,outdat);
        else
            error = [error err{:}];
            label = 'Error';
            icon = 'error';
            uiwait(msgbox(err(:),label,icon,'modal'));            
            return;
        end

    % Aqualog (HORIBA), G. GOLD dataset
    case 'Horiba_Aqualog'
        % Ex in columns and Em in rows, Ex in descending order (AquaLog)
        type = 3;
        % Aqualog's valid numeric range is 'A3' --> [0 2] para CSVREAD
        % eliminamos filas 1 y 2 donde aparecen datos NO numricos 
        % en fila 2 aparecen Ex que nos cargamos --> headers = [0 1]
        % como en CSVREAD el rango se epecifica numricamente desde [0 0]
        % (zero-based) es decir 'A1' --> [0 0] por eso 'A3' --> [0 2]
        % ver tambin READINEEMS
        validRange = [0 2]; % 'A3' (las dos primeras filas son de valores NO numricos)
        [range,~,err] = getrange(validFiles{1}, validRange);
        % [0 1]:  present for Em but not Ex - this is usual for Varian and AquaLog files once text are excluded from range
        headers = [0 1];
        display = 0;
        outdat = 0;
        if isempty(err)
            [X,Emmat,Exmat,filelist,~] = readineems(type,format,range,headers,display,outdat);
        else
            error = [error err{:}];
            label = 'Error';
            icon = 'error';
            uiwait(msgbox(err(:),label,icon,'modal'));            
            return;
        end

    % Varian Cary Eclipse (AGILENT)
    % A. HECHAVARRIA dataset
    case 'Agilent_VarianCaryEclipse'
%         help = [{'Please define just the numerical valid range for the Agilent Varian Cary Eclipse EEM'}
%             {'To get the valid range, just open the EEM file as an Excel spreadsheet'}
%             {'Take into account that he range is zero based, the ''1A1'' cell is the [0 0] reference'}];
%         label = 'Help';
%         icon = 'help';
%         uiwait(msgbox(help(:),label,icon,'modal'));

        % Ex in columns and Em in rows, with Em headers displayed in every second column (Varian)
        type = 2;
        validRange = [0 2]; % 'A3' (las dos primeras filas son de valores NO numricos) [startR startC endR endC]
%        validRange = [0 3 0 333];
        [range,~,err] = getrange(validFiles{1}, validRange);
        headers = [0 1]; % [0 1]:  present for Em but not Ex - this is usual for Varian and AquaLog files once text are excluded from range
        display = 0;
        outdat = 0;

        if isempty(err)
            [X,Emmat,Exmat,filelist,~] = readineems(type,format,range,headers,display,outdat);
        else
            error = [error err{:}];
            label = 'Error';
            icon = 'error';
            uiwait(msgbox(err(:),label,icon,'modal'));            
            return;
        end

    % FP8300 (JASCO),  TXT files
    % UCA (Universidad de Cdiz) dataset
    case 'Jasco_FP8300'
        [X,Emmat,Exmat,filelist,err] = readineems4jsc();
        if ~isempty(err)
            error = [error err{:}];
            label = 'Error';
            icon = 'error';
            uiwait(msgbox(err(:),label,icon,'modal'));            
            return;
        end
        
    %  F-2700 (HITACHI), XLS files
    % PATRICK GAUDET
    % MANC (School of  Civil and Environmental Engineering, Manhattan College) dataset
    case 'Hitachi_F2700'
        [X,Emmat,Exmat,filelist,err] = readineems4htiFL();
        if ~isempty(err)
            error = [error err{:}];
            label = 'Error';
            icon = 'error';
            uiwait(msgbox(err(:),label,icon,'modal'));            
            return;
        end
        
    otherwise
        %other EEM files
        label = 'Load of EEM files';
        info = strcat({ ...
            'The load of EEM files for the ' device ' is not implemented yet', ...
            'EEM files will NOT be loaded', ...
            });
        uiwait(msgbox(info,label,'modal'));
        error = [ ...
            error, ...
            {'ERROR: EEMlab:lodDsetEems:notImplementedOption'}, ...
            {'EEM files are NOT loaded'}, ...
            ];
        return;
        
end
cd(home);

%% === UPDATE DATASET
Ex = Exmat(1,:); % since the rows of Emmat and Exmat are each identical, we can extract one row of
Em = Emmat(:,1); % excitation wavelengths and one column of emission wavelengths that apply to all samples
units = 'AU';	% 'AU' Arbitrary units
                % 'QSE' Quinine Sulphate Equivalents
                % 'RU' Raman units
                % 'RFU' Relative Fluorescence units
newdata = assembledataset(X,Ex,Em,units,'filelist',filelist,[]);
newdata.header = stag; % samplelog header, to empair files with 'alignds'
newloads.rawEEMs = newdata; %we keep a copy of raw EEMs in the LOADS struct

end