function [ newloads, error ] = lodNormBlkRam( data, loads, samplelog )
%LODNORMBLKRAM loads Raman scans RU350 (excitation at 350 nm) or RU275
%(275 nm) for the later Raman normalization of blanks

%% === INIT
newloads = loads;
error = {};
rflag = false; % recursive flag for 'getfilesinfolder'
pflag = false; % prepend flag for 'getfilesinfolder'
dflag = true; % deblank flag for 'targetsinlist'
home = pwd; % get current path

% selected landa wavelength (275 or 350 nm)
contents = cellstr(loads.config.landa.string);
landa = str2double(contents{loads.config.landa.value});

%% === INFO
% label = 'Load of water RAMAN scans';
% info = strcat({ ...
%     ['Load of water Raman scans at excitation ' num2str(wl) ' nm'], ...
%     ['The blanks are normalized to RU' num2str(wl)], ...
%     '', ...
%     'A.L. Lawaetz, C.A. Stedmon', ...
%     '"Fluorescence Intensity Calibration Using the Raman Scatter Peak of Water"' ...
%     'Appl. Spectrosc., vol. 63, number 8, (2009) 936-940', ...
%     });
% uiwait(msgbox(info,label,'modal'));

%% === ERROR CHECKING
% get the RAMAN filelist from samplelog ('loglist')
headers = fieldnames(samplelog);
[tag,ok] = listdlg('Name','Select the metadata tag',...
    'PromptString','Select the tag name to load:',...
    'SelectionMode','single',...
    'ListSize',[160 250],...
    'ListString',headers);
if ~ok
    error = [error, {'ERROR: EEMlab:lodNormBlkRam:noSamplelogReference'}];
    return;
end
stag = headers{tag}; % selected tag
loglist = samplelog.(stag);

% select folder and get the CORRECTION filelist ('filelist')
title = 'Select the directory containing the set of RAMAN files to load';
folder = uigetdir('',title); % get path to selected folder
if folder == 0 %user selected cancel
    error = [error, {'ERROR: EEMlab:lodNormBlkRam:userSelectedCancel'}];
    return;
end
filelist = getfilesinfolder(folder,rflag,pflag);

% match the selected RAMAN dataset against the samplelog metadata list
[validFiles,outfle,~] = targetsinlist(loglist,filelist,dflag); % check if samplelog references are in filelist
if ~isempty(outfle) % NOT all the references in samplelog are IN the folder
    info = strcat({ ...
        'This folder does not contain the complete RAMAN dataset referred to in the ''SampleLog'' metadata', ...
        '1.- complete the dataset and repeat the load OR...', ...
        '2.- delete unmatched records from ''SampleLog'' and repeat the metainformation & loads', ...
        ' ', ...
        'List of RAMAN files NOT stored in folder:', ...
        outfle{:}, ...
        });
    label = 'Error';
    icon = 'error';
    uiwait(msgbox(info,label,icon,'modal'));
    error = [ ...
        error, ...
        {'ERROR: EEMlab:lodNormBlkRam:incompleteDataset'}, ...
        {'List of RAMAN files NOT stored in folder:'}, ...
        outfle{:}, ...
        ];
    return;
end
[~,outlog,~] = targetsinlist(filelist,loglist,dflag); % check if files in folder are referred to in samplelog
if ~isempty(outlog) % the folder contains extra files
    info = strcat({ ...
        'The folder contains RAMAN files with no reference in the ''SampleLog'' metadata file', ...
        '1.- delete unmatched files from folder and repeat the load', ...
        ' ', ...
        'List of RAMAN files NOT referenced in ''SampleLog'':', ...
        outlog{:}, ...
        });
    label = 'Error';
    icon = 'error';
    uiwait(msgbox(info,label,icon,'modal'));
    error = [ ...
        error, ...
        {'ERROR: EEMlab:lodNormBlkRam:manyFilesInFolder'}, ...
        {'List of RAMAN files NOT referenced in ''SampleLog'':'}, ...
        outlog{:}, ...
        ];
    return;
end

%% === FORMAT checking
[~,format] = strtok(validFiles{1},'.');
format = format(2:end);
contents = cellstr(loads.config.fluor.string);
device = contents{loads.config.fluor.value};
% DEVICES: QuantaMaster,SpexFluorog3,Aqualog,Fluoromax...
if (strcmpi(format,'txt') && strcmpi(device,'Horiba_QuantaMaster')) || ...
        (strcmpi(format,'csv') && strcmpi(device,'Horiba_SpexFluorolog3')) || ...
        (strcmpi(format,'csv') && strcmpi(device,'Horiba_Aqualog')) || ...
        (strcmpi(format,'csv') && strcmpi(device,'Horiba_FluoroMax')) || ...
        (strcmpi(format,'csv') && strcmpi(device,'Agilent_VarianCaryEclipse')) || ...
        (strcmpi(format,'txt') && strcmpi(device,'Jasco_FP8300'))
    lflag = device; % acquisition device flag for loads
else
    info = strcat({ ...
        ['The reference file format (' validFiles{1} ') do not matches with the format generated by the selected device'], ...
        'EEM files will NOT be loaded', ...
        });
    label = 'Error';
    icon = 'error';
    uiwait(msgbox(info,label,icon,'modal'));
    error = [ ...
        error, ...
        {'ERROR: EEMlab:lodNormBlkRam:wrongFileFormat'}, ...
        ];
    return;    
end

%% === DATA LOAD
cd(folder);
switch lflag
    %QuantaMaster TXT RAMAN scans
    case 'Horiba_QuantaMaster'
        [X,~,wave,filelist,err] =readinscans4qms();
        if ~isempty(err)
            error = [error err{:}];
            label = 'Error';
            icon = 'error';
            uiwait(msgbox(err(:),label,icon,'modal'));
            return;
        end
    %Other fluorometers CSV RAMAN scans: SPEX Fluorolog-3, Fluoromax,
    case {'Horiba_SpexFluorolog3', 'Horiba_FluoroMax'}
        type = 'Raman scans for EEMs';
        [range,~,err] = getrange(validFiles{1});
        display = 0;
        outdat = 0;
        if isempty(err)
            [X,~,wave,filelist] = readinscans(type,format,range,display,outdat);
        else
            error = [error err{:}];
            label = 'Error';
            icon = 'error';
            uiwait(msgbox(err(:),label,icon,'modal'));
            return;
        end
    % Horiba Aqualog CSV Raman scans
    case 'Horiba_Aqualog'
        % Aqualog's valid numeric range is 'A3' --> [0 2] para CSVREAD
        % eliminamos filas 1 y 2 donde aparecen datos NO numricos
        % en fila 2 aparecen Ex que nos cargamos --> headers = [0 1]
        % como en CSVREAD el rango se epecifica numricamente desde [0 0]
        % (zero-based) es decir 'A1' --> [0 0] por eso 'A3' --> [0 2]
        % ver tambin READINEEMS
        validRange = [0 2]; % 'A3' (las dos primeras filas son de valores NO numricos)
        [xls_range, csv_range, err] = getrange(validFiles{1}, validRange);
        if isempty(err)
            display = 0;
            outdat = 0;
            switch csv_range(2)
                case {0, 1}
                    % ERROR, no scans nor EEMs provided!
                case 2
                    % if TYPE == SCAN --> no other correction needed for the FORMAT variable in drEEM's READINSCANS
                    type = 'Aqualog scans';
                otherwise
                    % if TYPE == EEM  --> the FORMAT variable specifies which EEM columns has to be read in order to get the SCANS
                    % Ex in columns and Em in rows, with Em headers displayed in every second column (Varian)
                    type = 'Aqualog EEMs';
                    % take Ems from the 1st column and intensity values depending on the landa
                    % in original file Ex values are from top to down
                    % in data.Ex ex values are stored from down to top
                    % then to get the exact flie index we have to use the FLIP
                    % UP to DOWN (flipud) matlab function
                    index = find(flipud(data.Ex) == landa);
                    landa_index = 1 + index;
                    format = [format '_1_' num2str(landa_index)];
            end
            [X,~,wave,filelist] = readinscans(type,format,xls_range,display,outdat);
        else
            error = [error err{:}];
            label = 'Error';
            icon = 'error';
            uiwait(msgbox(err(:),label,icon,'modal'));
            return;
        end
        % Varian Cary Eclipse Raman scans
    case 'Agilent_VarianCaryEclipse'
        % 'A3' (las dos primeras filas son de valores NO numricos para VARIAN)
        validRange = [0 2];
        [xls_range, csv_range, err] = getrange(validFiles{1}, validRange);
        if isempty(err)
            display = 0;
            outdat = 0;
            switch csv_range(2)
                case {0, 1}
                    % ERROR, no scans nor EEMs provided!
                case 2
                    % if TYPE == SCAN --> no other correction needed for the FORMAT variable in drEEM's READINSCANS
                    type = 'VarianCary scans';
                otherwise
                    % if TYPE == EEM  --> the FORMAT variable specifies which EEM columns has to be read in order to get the SCANS
                    % Ex in columns and Em in rows, with Em headers displayed in every second column (Varian)
                    type = 'VarianCary EEMs';
                    % take Ems from the 1st column and intensity values depending on the landa
                    % in original file Ex values are from top to down
                    % in data.Ex ex values are stored from down to top
                    % then to get the exact flie index we have to use the FLIP
                    % UP to DOWN (flipud) matlab function
                    index = find(data.Ex == landa);
                    if isempty(index)
                        info = strcat({ ...
                            ['The EEM provided for Raman normalization does NOT include values to the normalization wavelength at ' num2str(landa) 'nm'], ...
                            'Dataset Raman normalization files will NOT be loaded', ...
                            });
                        label = 'Error';
                        icon = 'error';
                        uiwait(msgbox(info,label,icon,'modal'));
                        error = [ ...
                            error, ...
                            {'ERROR: EEMlab:lodNormDsRam:landaNotIncludedInFile'}, ...
                            ];
                        return;
                    else
                        landa_index = 2 * index;
                        format = [format '_1_' num2str(landa_index)];
                    end
            end
            [X,~,wave,filelist] = readinscans(type,format,xls_range,display,outdat);
        else
            error = [error err{:}];
            label = 'Error';
            icon = 'error';
            uiwait(msgbox(err(:),label,icon,'modal'));
            return;
        end

    otherwise
        %other CORRECTION files for Hitachi, Jasco
        label = 'Load of RAMAN scans for BLANK files';
        info = strcat({ ...
            'the load of Raman scans for the ' device ' is not implemented yet', ...
            'Raman scans will NOT be loaded', ...
            });
        icon = 'warn';
        uiwait(msgbox(info,label,icon,'modal'));
        error = [ ...
            error, ...
            {'ERROR: EEMlab:lodNormBlkRam:notImplementedOption'}, ...
            {'RAMAN scans for BLANK files are NOT loaded'}, ...
            ];
        return;
        
end
cd(home);

%% === UPDATE DATASET
T = alignds(samplelog,{data.header,data.filelist},{stag,filelist,X});
T = [wave; T]'; % add wavelength information (first column) to 'fdomcorrect' inputs
newloads.T = T;

end