function [ newdata, error ] = mdlsValModel( data )
%
% This uses the drEEM's SPLITDS and SPLITANALYSIS scripts

%% === INIT
newdata = data;
error = {};

%% === INFO
label = 'Split half analysis model validation';
info = strcat({ ...
    'Compare the Ex/Em overall model spectra for all the splits of a dataset', ...
    'The model is valid if splits contain independent datasets and components are congruent', ...
    'This processing uses the EEMlab''s SPLITVALIDATION011 script', ...
    'Type HELP SPLITVALIDATION011 in MATLAB''s console for more info'
    });
uiwait(msgbox(info,label,'modal'));

%% === ERROR CHECKING
if ~isfield(data,'modelNames') || isempty(data.modelNames)
    info = strcat({ ...
        'The dataset has not been still modeled', ...
        'Please create an exploratory or refined model first', ...
        });
    label = 'Error';
    icon = 'error';
    uiwait(msgbox(info,label,icon,'modal'));
    error = [error, {'ERROR: EEMlab:mdlsValModel:noModelsToProcess'}];
    return;
elseif ~isfield(data,'Split') % no splitted models
    info = strcat({ ...
        'The splits of the dataset have not still been modeled', ...
        'Please create models for every one of the splits first', ...
        });
    label = 'Error';
    icon = 'error';
    uiwait(msgbox(info,label,icon,'modal'));
    error = [error, {'ERROR: EEMlab:mdlsValModel:noModelsToProcess'}];
    return;
elseif isempty(data.modelNames(~cellfun(@isempty,strfind(data.modelNames,'- to validate')))) % splitted models but all of them VALID or NOT VALID...
    info = strcat({ ...
        'Validation has alredy been applied on ALL the refined models', ...
        });
    label = 'Help';
    icon = 'help';
    uiwait(msgbox(info,label,icon,'modal'));
end

%% === DATA LOAD
% first at all we delete OLD validations
datafields = fieldnames(data);
ind = ~cellfun(@isempty,regexp(datafields,'Val')); %model indexes
fields_to_delete = datafields(ind);
data = rmfield(data,fields_to_delete);
% Configuring validation of the models
user_info = mdlsValModelFig(data);

if isempty(user_info)
    error = [error, {'ERROR: EEMlab:mdlsValModel:userSelectedCancel'}];
    return;
% splitted models (validated or NOT) are REQUIRED
elseif isempty(data.modelNames(~cellfun(@isempty,(strfind(data.modelNames,['Model' num2str(user_info.f) ' (splitted'])))))
    info = strcat({ ...
        ['There are no ' num2str(user_info.f) '-component models for the dataset splits'], ...
        'Please model the splitted dataset first', ...
        });
    label = 'Error';
    icon = 'error';
    uiwait(msgbox(info,label,icon,'modal'));
    error = [error, {'ERROR: EEMlab:mdlsValModel:noModelsToProcess'}];
    return;
else
    % now we do the NEW validation
    if isempty(user_info.overall) % SHA model
        warning('off');
        data = splitvalidation012(data,user_info.f,user_info.comparisons,user_info.splitnames,[],user_info.p);
        warning('on');
    else
        data = splitvalidation012(data,user_info.f,user_info.comparisons,user_info.splitnames,data,user_info.p);
    end
% if finishes OK, delete the MATLAB temporal file
if exist(fullfile(pwd,'temp.mat'),'file') == 2
    delete(fullfile(pwd,'temp.mat'));
end

%% === UPDATE DATASET
results = cellstr(data.Val_Result);
if (size(results,1) == 1) && ~isempty(strfind(results{:},'Validated for all comparisons')) % ALL the models are VALID
    label = 'Validation of the overall model';
    info = strcat({ ...
        'The overall model is VALID for ALL comparisons', ...
        ['Criterion for spectral match p = ' num2str(user_info.p)], ...
        '', ...
        results{:}, ...
        });
    uiwait(msgbox(info,label,'help','modal'));
    if strfind(user_info.smodel,'to validate')
        data.modelNames{~cellfun(@isempty,(strfind(data.modelNames,user_info.smodel)))} = ...
            strrep(user_info.smodel,'to validate','VALID');
    elseif strfind(user_info.smodel,'NOT VALID')
        data.modelNames{~cellfun(@isempty,(strfind(data.modelNames,user_info.smodel)))} = ...
            strrep(user_info.smodel,'NOT VALID','VALID');
    end
elseif sum(~cellfun(@isempty,strfind(results,'-Validated'))) > 0 % some model is VALID
    label = 'Validation of the overall model';
    info = strcat({ ...
        'The overall model is valid at least for ONE of the splits', ...
        ['Criterion for spectral match p = ' num2str(user_info.p)], ...
        '', ...
        results{:}, ...
        });
    uiwait(msgbox(info,label,'help','modal'));
    if strfind(user_info.smodel,'to validate')
        data.modelNames{~cellfun(@isempty,(strfind(data.modelNames,user_info.smodel)))} = ...
            strrep(user_info.smodel,'to validate','VALID');
    elseif strfind(user_info.smodel,'NOT VALID')
        data.modelNames{~cellfun(@isempty,(strfind(data.modelNames,user_info.smodel)))} = ...
            strrep(user_info.smodel,'NOT VALID','VALID');
    end
else % NONE of the splits generates a VALID model
    label = 'Validation of the overall model';
    info = strcat({ ...
        'The overall model is NOT VALID', ...
        ['Criterion for spectral match p = ' num2str(user_info.p)], ...
        '', ...
        results{:}, ...
        });
    uiwait(msgbox(info,label,'warn','modal'));
    if strfind(user_info.smodel,'to validate')
    data.modelNames{~cellfun(@isempty,(strfind(data.modelNames,user_info.smodel)))} = ...
        strrep(user_info.smodel,'to validate','NOT VALID');
    elseif strfind(user_info.smodel,'VALID')
        data.modelNames{~cellfun(@isempty,(strfind(data.modelNames,user_info.smodel)))} = ...
            strrep(user_info.smodel,'VALID','NOT VALID');
    end
end
newdata = data;

%%%%%%%%%%%%%%%%%%%%%%%%% THIS IS A MORE RESTRICTIVE CONDITION...
% %% === UPDATE DATASET
% results = cellstr(data.Val_Result);
% negative = sum(~cellfun(@isempty,strfind(results,'Not')));
% if negative > 0 % NOT VALIDATED model
%     label = 'Validation of the overall model';
%     info = 'The selected PARAFAC model is NOT VALID';
%     uiwait(msgbox(info,label,'warn','modal'));
%     data.modelNames{~cellfun(@isempty,(strfind(data.modelNames,user_info.smodel)))} = ...
%         strrep(user_info.smodel,'to validate','NOT VALID');
% else % VALID model
%     label = 'Validation of the overall model';
%     info = 'The selected PARAFAC model is VALID';
%     uiwait(msgbox(info,label,'help','modal'));
%     data.modelNames{~cellfun(@isempty,(strfind(data.modelNames,user_info.smodel)))} = ...
%         strrep(user_info.smodel,'to validate','VALID');
% end
% newdata = data;

end