function [ error ] = pltMdlEres2( data )
%

%% === INIT
error = {};

%% === ERROR CHECKING
if ~isfield(data,'modelNames') || isempty(data.modelNames)
    info = strcat({ ...
        'The dataset has not been still modeled', ...
        'Please create two of the exploratory, refined or splitted models first', ...
        });
    label = 'Error';
    icon = 'error';
    uiwait(msgbox(info,label,icon,'modal'));
    error = [error, {'ERROR: EEMlab:pltMdlEres2:noModelsToProcess'}];
    return;
end

%% === INFO
label = 'Compare error residuals';
info = strcat({ ...
    'Compare the sample error residuals for TWO selected models', ...
    'In order to ensure the comparison between models with', ...
    'the same number of samples the selected models must belong to:', ...
    '(i) the exploratory or refined categories or', ...
    '(ii) to the SHA category', ...
    'This processing uses the drEEM''s EEMVIEW script', ...
    'Type HELP EEMVIEW in MATLAB''s console for more info', ...
    });
uiwait(msgbox(info,label,'modal'));

%% === DATA LOAD
name = 'Compare error residuals';
models = data.modelNames;
[selected,ok] = listdlg('Name',name,...
    'PromptString','Select the models to compare', ...
    'SelectionMode','multiple',...
    'ListSize',[180 180],...
    'ListString',models);
if length(selected) ~= 2
    info = strcat({ ...
        'You have to select only TWO models from the list', ...
        });
    label = 'Error';
    icon = 'error';
    uiwait(msgbox(info,label,icon,'modal'));
    error = [error, {'ERROR: EEMlab:pltMdlEres2:numberOfArgumentsWrong'}];
    return;
elseif ~ok
    error = [error, {'ERROR: EEMlab:pltMdlEres2:userSelectedCancel'}];
    return;
end
smodels = models(selected);
myfactors = nan(1,length(smodels));
EXlist = false(length(smodels),1); % list of Exploratory (OUTLIERTEST) models
RFlist = false(length(smodels),1); % list Refined (RANDINITANAL) models
SPlist = false(length(smodels),1); % list Validated (SPLITDS, SPLITANALYSIS, SPLITVALIDATION) models
for i = 1 : length(smodels)
    tok = strtok(smodels{i},'Model');
    myfactors(i) = str2double(tok(1));
    if strfind(smodels{i},'exploratory')
        EXlist(i,1) = true;
    elseif strfind(smodels{i},'refined')
        RFlist(i,1) = true;
    elseif strfind(smodels{i},'splitted')
        SPlist(i,1) = true;
    end
end

% now we load the SHA split to plot (Name of the halves)
if sum(SPlist) == 2
    names = [{'A'},{'B'},{'C'},{'D'},{'E'},{'F'},{'G'},{'H'},{'I'}, ...
        {'J'},{'K'},{'L'},{'M'},{'N'},{'O'},{'P'},{'Q'},{'R'},{'S'},{'T'},{'U'}, ...
        {'V'},{'W'},{'X'},{'Y'},{'Z'}];
    splitnames = cell(1,data.Split_NumAfterCombine);
    for i = 1 : data.Split_NumAfterCombine
        splitnames{i} = ['[' strjoin(names(str2num(data.Split_Combinations{i}))) ']'];
    end
    name = 'Select the splits (SHA halves)';
    [ssplit,ok] = listdlg('Name',name,...
        'PromptString','Select the dataset split', ...
        'SelectionMode','single',...
        'ListSize',[100 100],...
        'ListString',splitnames);
    if ~ok
        error = [error, {'ERROR: EEMlab:pltMdlSpecLod:userSelectedCancel'}];
        return;
    end
end

%% === PLOTS
erraxis = 0.05;
if sum(EXlist|RFlist) == 2
    compare2models(data,myfactors(1),myfactors(2),erraxis);
elseif sum(SPlist) == 2
    compare2models(data.Split(ssplit),myfactors(1),myfactors(2),erraxis);
else  % we can not compare models with different number of samples!
    label = 'Compare error residuals';
    info = strcat({ ...
        'The models to compare do NOT have the same number of samples', ...
        'In order to ensure the comparison between models with', ...
        'the same number of samples the selected models must belong to:', ...
        '(i) the exploratory or refined categories or', ...
        '(ii) the splitted category', ...
        });
    label = 'Error';
    icon = 'error';
    uiwait(msgbox(info,label,icon,'modal'));
    error = [error, {'ERROR: EEMlab:pltMdlEres2:noModelsToProcess'}];
    return;
end


% ====== REVIEW this!!!!
% ====== is much better to use EEMVIEW instead of COMPARE2MODELS!!!
%
% datafield = {'X',smodels{1},'error_residuals', ...
%     'X', smodels{2},'error_residuals'};
% plotlayout = [2 3];
% istart = [];
% plotview = [];
% datalabel = {'sample',smodels{1},'error for sample', ...
%     'sample',smodels{2},'error for sample'};
%  axeslimits = [1 1 0.1 1 1 0.1];
% rotate = [];
% colorbar = 'colorbar';
% ncontours = [];
% plotsize = [];
% eemview({data,data,data,data},datafield,plotlayout,istart, ...
%     plotview,datalabel,axeslimits,rotate, ...
%     colorbar,ncontours,plotsize);

%% === UPDATE DATASET

end