function [ error ] = pltMdlMapCmp( data )
%

%% === INIT
error = {};

%% === ERROR CHECKING
if ~isfield(data,'modelNames') || isempty(data.modelNames)
    info = strcat({ ...
        'The dataset has not been still modeled', ...
        'Please create tyhe exploratory, refined or splitted models first', ...
        });
    label = 'Error';
    icon = 'error';
    uiwait(msgbox(info,label,icon,'modal'));
    error = [error, {'ERROR: EEMlab:pltMdlMapCmp:noModelsToProcess'}];
    return;
end

%% === INFO
% label = 'Map model components';
% info = strcat({ ...
%     'Scatter plot of Ex/Em wavelengths for the maximum intensity of all the components in the model', ...
%     '', ...
%     'W. Chen, P. Westerhoff, J.A. Leenheer & K. Booksh', ...
%     '"Fluorescence Excitation-Emission Matrix Regional Integration to Quantify Spectra for Dissolved Organic Matter"', ...
%     'Environmental Science & Technology, 2003, 37, 5701 - 5710', ...
%     });
% uiwait(msgbox(info,label,'modal'));

%% === DATA LOAD
name = 'Map model components';
models = data.modelNames;
[selected,ok] = listdlg('Name',name,...
    'PromptString','Select a model', ...
    'SelectionMode','single',...
    'ListSize',[180 180],...
    'ListString',models);
if ~ok
    error = [error, {'ERROR: EEMlab:pltMdlMapCmp:userSelectedCancel'}];
    return;
end
smodel = models{selected};
tok = strtok(smodel,'Model');
myfactors = str2double(tok(1));

% now we load the SHA split to plot (Name of the halves)
if strfind(smodel,'splitted')
    names = [{'A'},{'B'},{'C'},{'D'},{'E'},{'F'},{'G'},{'H'},{'I'}, ...
        {'J'},{'K'},{'L'},{'M'},{'N'},{'O'},{'P'},{'Q'},{'R'},{'S'},{'T'},{'U'}, ...
        {'V'},{'W'},{'X'},{'Y'},{'Z'}];
    splitnames = cell(1,data.Split_NumAfterCombine);
    for i = 1 : data.Split_NumAfterCombine
        splitnames{i} = ['[' strjoin(names(str2num(data.Split_Combinations{i}))) ']'];
    end
    name = 'Select the splits (SHA halves)';
    [ssplit,ok] = listdlg('Name',name,...
        'PromptString','Select the dataset split', ...
        'SelectionMode','single',...
        'ListSize',[100 100],...
        'ListString',splitnames);
    if ~ok
        error = [error, {'ERROR: EEMlab:pltMdlCompcorr:userSelectedCancel'}];
        return;
    end
end

%now we get plot values
if ~isempty(strfind(smodel,'exploratory')) || ~isempty(strfind(smodel,'refined'))
    exAxe = data.Ex;
    emAxe = data.Em;
    values = data.(['Model' num2str(myfactors)]);    
elseif strfind(smodel,'splitted')
    exAxe = data.Split(ssplit).Ex;
    emAxe = data.Split(ssplit).Em;
    values = data.Split(ssplit).(['Model' num2str(myfactors)]);    
end

emScans = values{2}; % values{2} --> nEm x nComp (emission LOADINGS)
exScans = values{3}; % values{3} --> nEx x nComp (excitation LOADINGS)
%concentration = values{1}; % values{1} --> nSamp x nComp (concentration SCORES)
exWl = NaN(1,myfactors);
emWl = NaN(1,myfactors);
for i = 1 : myfactors
    exWl(i) = exAxe(exScans(:,i) == max(exScans(:,i)));
    emWl(i) = emAxe(emScans(:,i) == max(emScans(:,i)));
end

%% === PLOTS
% xAxe = 280:550; % emission wavelengths in nm
% yAxe = 200:400; % excitation wavelengths in nm
% hborder1 = 250*ones(1,length(280:550));
% vborder1 = 330*ones(1,length(200:250));
% vborder2 = 380*ones(1,length(200:400));
% 
% figure('NumberTitle','off', 'Name',[smodel ' components map'], ...
%     'WindowStyle','normal');
% datacursormode on;
% hold on;
% % === regions' borders
% plot(xAxe,hborder1,':b', 'LineWidth',2);
% plot(vborder1,200:250,':b', 'LineWidth',2);
% plot(vborder2,200:400,':b', 'LineWidth',2);
% % === regions' legends
% text(305,250,{'Region I', 'Aromatic Protein'}, ...
%     'Color','b', 'FontSize',7, 'HorizontalAlignment','center', ...
%     'VerticalAlignment','top'); % center, up
% text(355,250,{'Region II', 'Aromatic ProteinII'}, ...
%     'Color','b', 'FontSize',7, 'HorizontalAlignment','center', ...
%     'VerticalAlignment','top'); % center, up
% text(465,250,{'Region III', 'Fulvic acid-like'}, ...
%     'Color','b', 'FontSize',7, 'HorizontalAlignment','center', ...
%     'VerticalAlignment','top'); % center, up
% text(330,400,{'Region IV', 'Soluble microbial', 'by-product-like'}, ...
%     'Color','b', 'FontSize',7, 'HorizontalAlignment','center', ...
%     'VerticalAlignment','top'); % center, up
% text(465,400,{'Region V', 'Humic acid-like'}, ...
%     'Color','b', 'FontSize',7, 'HorizontalAlignment','center', ...
%     'VerticalAlignment','top'); % center, up
% % === components (scatter plot)
% scatter(emWl,exWl,25,'r','o','filled');
% % ===
% hold off;
% grid on;
% axis([xAxe(1) xAxe(end) yAxe(1) yAxe(end)]);
% xlabel('Emission (nm)');
% ylabel('Excitation (nm)');

str_xAxe = min(200,min(exWl));
end_xAxe = max(400,max(exWl));
str_yAxe = min(280,min(emWl));
end_yAxe = max(550,max(emWl));
xAxe = str_xAxe:end_xAxe; % excitation wavelengths in nm
yAxe = str_yAxe:end_yAxe; % emission wavelengths in nm
vborder1 = 250*ones(1,length(str_yAxe:end_yAxe));
hborder1 = 330*ones(1,length(str_xAxe:250));
hborder2 = 380*ones(1,length(str_xAxe:end_xAxe));

figure('NumberTitle','off', 'Name',[smodel ' components map'], ...
    'WindowStyle','normal');
datacursormode on;
hold on;
% === regions' borders
plot(vborder1,yAxe,':b', 'LineWidth',2);
plot(200:250,hborder1,':b', 'LineWidth',2);
plot(200:400,hborder2,':b', 'LineWidth',2);
% === regions' legends
text(225,330,{'Region I', 'Aromatic Protein'}, ...
    'Color','b', 'FontSize',7, 'HorizontalAlignment','center', ...
    'VerticalAlignment','top'); % center, up
text(225,380,{'Region II', 'Aromatic ProteinII'}, ...
    'Color','b', 'FontSize',7, 'HorizontalAlignment','center', ...
    'VerticalAlignment','top'); % center, up
text(225,550,{'Region III', 'Fulvic acid-like'}, ...
    'Color','b', 'FontSize',7, 'HorizontalAlignment','center', ...
    'VerticalAlignment','top'); % center, up
text(325,380,{'Region IV', 'Soluble microbial', 'by-product-like'}, ...
    'Color','b', 'FontSize',7, 'HorizontalAlignment','center', ...
    'VerticalAlignment','top'); % center, up
text(325,550,{'Region V', 'Humic acid-like'}, ...
    'Color','b', 'FontSize',7, 'HorizontalAlignment','center', ...
    'VerticalAlignment','top'); % center, up
% === components (scatter plot)
%scatter(exWl,emWl,25,'r','o','filled');
%gscatter(exWl,emWl,1:myfactors)
for i = 1 : myfactors
    plot(exWl(i),emWl(i), 'ro', 'MarkerFaceColor','r');
    text(exWl(i),emWl(i),['  ' num2str(i)], 'HorizontalAlignment','left');
end

% ===
hold off;
grid on;
axis([xAxe(1) xAxe(end) yAxe(1) yAxe(end)]);
xlabel('Excitation (nm)');
ylabel('Emission (nm)');
title([num2str(myfactors) '-components Model']);

%% === UPDATE DATASET

end