function [ hdl, error ] = plotEem( exAxe, emAxe, intValues, intUnits, dim, spoint, varargin )
%
% PLOTEEM plots the selected EEM sample and marks the selected point
%
% EXAXE         excitation wavelengths
% EMAXE         emission wavelengths
% INTVALUES     fluorescence intensity values
% INTUNITS      fluorescence intensity units
% DIM           dimensions to be plotted (values '2D', '3D')
% SPOINT        selected point coordinates [exAxe(i) emAxe(i)] to be marked
% VARARGIN{1}   selected axes handles to plot the EEM in
% VARARGIN{2}   selected axes handles to plot the EEM's COLORBAR in
%
% HDL           handle (figure or axes) in where the plot is displayed
% ERROR         error log missage

%% INIT
error = {};

%% PARSE inputs & ERROR checking
switch nargin
    case 1
    case 2
    case 3
    case 4
    case 5
        error = [error, ...
            {'ERROR: EEMlab:plotEem:invalidNumberOfInputArguments'}];
        return;
    case 6
        hdl = figure('NumberTitle','off', 'Name','EEM for the selected sample', ...
            'WindowStyle','normal');
        datacursormode on;
    case 7
        hdl = varargin{1}; % axes handle
        if strcmp(get(hdl,'type'),'axes')
            axes(hdl);
        else
            error = [error, ...
                {'ERROR: EEMlab:plotEem:invalidAxesType'}];
            return;
        end
    otherwise
        error = [error, ...
            {'ERROR: EEMlab:plotEem:invalidNumberOfInputArguments'}];
        return;
end

%% === PLOT
switch upper(dim)
    
    case '2D'
        contourf(exAxe,emAxe,intValues);
        if ~isempty(spoint)
            hold on;
            plot(exAxe,spoint(2)*ones(size(exAxe)), ':r', 'LineWidth',2);
            plot(spoint(1)*ones(size(emAxe)),emAxe, ':r', 'LineWidth',2);
            plot(spoint(1),spoint(2), '+w', 'MarkerSize',8, 'LineWidth',2);
            hold off;
        end
        axis([exAxe(1) exAxe(end) emAxe(1) emAxe(end)]);
        grid on;
        colormap('Parula');
        title(['Fluorescence Intensity (' intUnits ')'], 'FontSize',10, 'FontWeight','normal');
        %xlabel('excitation (nm)');
        %ylabel('emission (nm)');
        set(hdl, 'XTickLabel',[], 'YTickLabel',[]); % delete ticks in X & Y axes
        colorbar('eastoutside');
        set(gca, 'CLim',[min(intValues(:)) max(intValues(:))]);
        
    case '3D'
        ex_ind = exAxe == spoint(1);
        em_ind = emAxe == spoint(2);
        sIntVal = intValues(em_ind,ex_ind); % intensity value for the selected point
        mesh(exAxe,emAxe,intValues);
        %surf(exAxe,emAxe,intValues);
        if ~isempty(spoint)
            hold on;
            plot3(spoint(1),spoint(2),sIntVal, '+r', 'MarkerSize',8, 'LineWidth',1);
            hold off;
        end
        axis([exAxe(1) exAxe(end) emAxe(1) emAxe(end) min(intValues(:)) 1.1*max(intValues(:))]);
        grid on;
        %xlabel('excitation (nm)');
        %ylabel('emission (nm)');
        zlabel(['fluorescence intensity (' intUnits ')']);
        set(hdl, 'XTickLabel',[], 'YTickLabel',[], 'ZTickLabel',[]); % delete ticks in X, Y & Z axes
        colorbar('eastoutside');
        set(gca, 'CLim',[min(intValues(:)) max(intValues(:))]);
        
    otherwise
        error = [error, ...
            {'ERROR: EEMlab:plotEem:numberOfPlotDimensionsWrong'}];
        return;
end

end