function [ hdl, error ] = plotScans( exAxe, emAxe, intValues, intUnits, spoint, varargin )
%
% PLOTSCANS plots the excitation/emission scans for the selected EEM
% intensity value in SPOINT
%
% EXAXE         excitation wavelengths
% EMAXE         emission wavelengths
% INTVALUES     fluorescence intensity values
% INTUNITS      fluorescence intensity units
% SPOINT        selected point coordinates [exAxe(i) emAxe(i)]
% VARARGIN{1}   selected axes handles to plot in
%
% HDL           handle (figure or axes) in where the plot is displayed
% ERROR         error log missage

%% INIT
error = {};

%% PARSE inputs & ERROR checking
switch nargin
    case 1
    case 2
    case 3
    case 4
        error = [error, ...
            {'ERROR: EEMlab:plotScans:invalidNumberOfInputArguments'}];
        return;
    case 5
        hdl = figure('NumberTitle','off', 'Name','Ex/Em scans for the selected sample', ...
            'WindowStyle','normal');
        datacursormode on;
    case 6
        hdl = varargin{1}; % axes handle
        if strcmp(get(hdl,'type'),'axes')
            axes(hdl);
        else
            error = [error, ...
                {'ERROR: EEMlab:plotScans:invalidAxesType'}];
            return;
        end
    otherwise
        error = [error, ...
            {'ERROR: EEMlab:plotScans:invalidNumberOfInputArguments'}];
        return;
end

%% === DATA
% getting VALUES for the selected point (SPOINT)
ex_ind = exAxe == spoint(1);
em_ind = emAxe == spoint(2);
exInt = intValues(em_ind,:)'; % excitation scan (we fix emission to the selected point)
emInt = intValues(:,ex_ind); % emission scan (we fix excitation to the selected point)


%% === PLOT
plot(exAxe,exInt, '-b', 'LineWidth',2);
hold on;
plot(emAxe,emInt, ':b', 'LineWidth',2);
hold off;
% environment
legend(['Ex scan (' num2str(spoint(2)) ' nm)'],['Em scan (' num2str(spoint(1)) ' nm)']);
axis([min([exAxe;emAxe]) max([exAxe;emAxe]) min([exInt;emInt]) 1.1*max([exInt;emInt])]);
set(gca, 'FontSize',8, 'FontWeight','normal');
grid on;
%title(['Ex scan at ' num2str(spoint(2)) ' nm - Em scan at ' num2str(spoint(1)) ' nm'], 'FontSize',8, 'FontWeight','normal');
xlabel('wavelength (nm)', 'FontSize',8, 'FontWeight','normal');
ylabel(['fluorescence intensity (' intUnits ')'], 'FontSize',8, 'FontWeight','normal');

end