function Xc=fdomcorrect024_ife(X,Ex,Em,Emcor,Excor,W,varargin)
%
% This is the EEMlab version of the drEEM's FDOMCORRECT function
% FDOMCORRECT012 improves the spectral correction. It applies diferent
% corrections to each sample as it is described in the SampleLog file
% FDOM: Fluorescent Dissolved Organic Matter
%
% USAGE:
%     [XcRU Arp IFCmat BcRU XcQS QS_RU]=fdomcorrect(X,Ex,Em,Emcor,Excor,W,RamOpt,A,B,T,Q,Qw);
%
% INPUTS
%      Matrix sizes indicated by (Rows x Columns) in 2D case or (Samples x Rows x Columns) in 3D case.
%
% 	  Ex:	1D row vector of excitation wavelengths corresponding to EEMs.
% 	  Em:	1D row vector of emission wavelengths corresponding to EEMs.
%  	   X:	is either of (a) or (b):
%           (a) A 3D data cube of EEMs (NO HEADERS, i.e. samples x Em x Ex).
%           (b) a data structure DS with the EEMs in DS.X and the filenames in DS.filelist
%  Emcor:	2D matrix ([wavelength; samples x correction factors]).
%           First ROW is wavelength. Each sample has its own spectral 
%           correction scan in a ROW
%  Excor:	2D matrix ([wavelength samples x correction factors]).
%           First row is wavelength. Each sample has its own spectral
%           correction scan in a ROW
% 	   W:	is either of (a) or (b):
%       	(a) a single water Raman emission scan (Ex==landa) applied to all samples; first row is wavelength;
%       	(b) 2D matrix (samples x Em) of water Raman scans (Ex==landa) for corresponding samples; 
%               first row is wavelength.
% RamOpt:	options (user-defined excitation and emission wavelengths, to nearest 0.5nm) 
%           for calculating Raman areas in the form: [landa Em1 Em2]. 
%           If RamOpt=[], the default values [350 381 426] are used.
%           RamOpt(1)= landa; excitation wavelength corresponding to the emission scan used for calculating Raman areas. 
%           RamOpt(2)= first emission wavelength in the integration range
%           RamOpt(3)= last emission wavelength in the integration range
%
% IF performing INNER FILTER CORRECTION using the Absorbance method, otherwise A=[]:
% 		A:	2D matrix (samples x wavelengths) of Absorbances (decadal form, 1 cm cell); first row is wavelength.
%
% IF performing BLANK SUBTRACTION, otherwise B=[],T=[]:
% 		B:	is either of (a) or (b):
%           (a) single blank EEM (Em x Ex) to be used for all samples;
%           (b) 3D data cube (samples x Em x Ex) of fluorescence intensities in blanks matched with samples.
% 		T: is either of (a) or (b):
%           (a) T=[] indicating that T is to be automatically extracted from B at Ex=landa;
%           (b) 2D matrix (samples x Em) of water Raman scans(Ex==landa) for blanks; first row is wavelength.
%
% IF performing QUININE SULFATE (QS) CALIBRATION, otherwise Q=[],Qw=[]:
% 		Q:	is either of (a) or (b):
%       	(a) a scalar representing the slope (uncorrected) of a linear QS dilution series 
%               at 350/450 nm applied to all samples in the dataset;
%       	(b) a vector of slopes (uncorrected) of QS dilution series corresponding with each sample.
% 		Qw:	is either of (a) or (b):
%       	(a) a water Raman emission scan (Ex==landa) collected on same day as the QS 
%				dilution series; first row is wavelength;
%       	(b) 2D matrix (samples x Em) of water Raman scans(Ex==landa) corresponding 
%               to the vector of QS blanks; in which the first row is wavelength.
%
% OUTPUTS:
% 	XcRU:	3D matrix of corrected EEMs in Raman units, including inner filter correction and blank subtraction steps, if applied.
% 	Arp:	vector of Raman Areas in arbitrary units.
% 	IFCmat:	3D matrix of inner filter correction factors.
% 	BcRU:	3D matrix of corrected blanks.
% 	XcQS:	3D matrix of corrected EEMs in QS units, including inner filter correction and blank subtraction steps, if applied.
% 	QS_RU:	vector of conversion factors between QS and RU calibrated EEMs.
%
% EXAMPLES:
% The simplest correction method requires 6 inputs and 2 outputs to produce spectrally corrected EEMs in RU units 
% There are no inner filter correction, blank subtraction or QS calibration steps:
% [Xc Arp]=fdomcorrect(X,Ex,Em,Emcor,Excor,W);   %spectral correction in RU units with landa = 350 nm
%
% if performing additional or non-default correction steps, all 12 input variables and 6 output variables must be specified 
% and irrelevant variables will be outputed as NaN. For example:
% [XcRU Arp IFCmat BcRU XcQS QS_RU]=fdomcorrect(X,Ex,Em,Emcor,Excor,W,[],A,[],[],[],[]);  %inner filter correction
% [XcRU Arp IFCmat BcRU XcQS QS_RU]=fdomcorrect(X,Ex,Em,Emcor,Excor,W,[],A,B,[],[],[]);   %inner filter correction, blank subtraction
% [XcRU Arp IFCmat BcRU XcQS QS_RU]=fdomcorrect(X,Ex,Em,Emcor,Excor,W,[],[],B,[],Q,Qw);   %blank subtraction, RU and QS normalization
% [XcRU Arp IFCmat BcRU XcQS QS_RU]=fdomcorrect(X,Ex,Em,Emcor,Excor,W,[280 300 325],[],[],[],Q,Qw);  %RU, QSE units with landa = 280 nm
%
% Notice:
% This mfile is part of the drEEM toolbox. Please cite the toolbox
% as follows:
%
% Murphy K.R., Stedmon C.A., Graeber D. and R. Bro, Fluorescence
%     spectroscopy and multi-way techniques. PARAFAC, Anal. Methods, 
%     5, 6557-6566, 2013. DOI:10.1039/c3ay41160e. 
%
% fdomcorrect: Copyright (C) 2014 Kathleen R. Murphy
% $ Version 0.2.0 $ June 2014 $ Second Release
% $ Version 0.1.0 $ September 2013 $ First Release
% $ Updated from FDOMcorr toolbox ver. 1.8
% fdomcorrect012: is a version of fdomcorrect $ October 2016 $ reviewed by Pau Mic

%
% Copyright (C) 2014 KR Murphy,
% Water Environment Technology
% Chalmers University of Technology
% Department of Civil and Environmental Engineering
% Water Environment Technology
% Sven Hultins gata 8
% 412 96 Gteborg
% murphyk@chalmers.se

clc;
format bank;
narginchk(6,12);

% Extract EEMs and filelist from a data structure
if isstruct(X)
    try
        X=X.X;
    catch
        error('fdomcorrect012:EEMs','Did not locate DS.X and DS.filelist in the input EEM data structure')
    end
end

%inputs
if nargin==6
    landa=350;
else
    if nargin<12
        error('function requires either 6 or 12 input variables.');
    elseif  nargin==12
        RamOpt=varargin{1};
        A=varargin{2};
    end
    if isempty(RamOpt)
        landa=350;
    else
        landa=RamOpt(1);
    end
end

%% Step 0: Pretreatment: Resize files as needed
fprintf('\n\n')
fprintf('Step 0: Checking input files...\n')
N_ex=max(size(Ex));
N_em=max(size(Em));
Em=CheckMatrixDim(Em,N_em,1,[],'Em');
Ex=CheckMatrixDim(Ex,1,N_ex,[],'Ex');
N_samples=size(X,1);  %or: N_samples=max(size(L));
X=CheckMatrixDim(X,N_samples,N_em,N_ex,'X');

% Obtain water Raman Scans for Samples
fprintf('  obtaining water Raman scans, W...\n')
if min(size(W))==2 % use a single Raman scan for all samples
   W=CheckMatrixDim(W,2,[],[],'W');
   W=[W(1,:)'  W(2,:)'*ones(1,N_samples)]';
end
W=CheckMatrixDim(W,N_samples+1,[],[],'W');

%Obtain correction files and resize as necessary
doplots=false;
fprintf('  obtaining and resizing correction files...\n')
fprintf('  matching EmcorX & Em \n');
EmcorX = Em;
for i = 2 : N_samples+1 %We have 2D matrix, each EEM with an specific correction file
    EmcorXscan=MatchWave([Emcor(:,1) Emcor(:,i)],Em,doplots); %Resize EmcorX to match Em,
    EmcorX(:,i) = EmcorXscan(:,2);
end
EmcorX=CheckMatrixDim(EmcorX,N_em,2,[],'Emcor for EEM');

fprintf('  matching ExcorX & Ex \n');
ExcorX = Ex';
for i = 2 : N_samples+1 %We have 2D matrix, each EEM with an specific correction file
    ExcorXscan=MatchWave([Excor(:,1) Excor(:,i)],Ex,doplots); %Resize EmcorX to match Em,
    ExcorX(:,i) = ExcorXscan(:,2);
end
ExcorX=CheckMatrixDim(ExcorX,N_ex,2,[],'Excor for EEM');

fprintf('  matching EmcorR & W \n');
wave = W(1,:)'; % take wavelengths from Raman scans
EmcorR = wave;
for i = 2 : N_samples+1 %We have 2D matrix, each EEM with an specific correction file
    EmcorRscan=MatchWave([Emcor(:,1) Emcor(:,i)],wave,doplots); %Resize EmcorX to match Em,
    EmcorR(:,i) = EmcorRscan(:,2);
end
EmcorR=CheckMatrixDim(EmcorR,size(W,2),2,[],'Emcor for Raman Scan');

fprintf('  looking for Excorlanda \n');
Excorlanda=Excor(Excor(:,1)==landa,2:end);
if isempty(Excorlanda)
    error('Could not locate ex=landa in correction files');
end

%% Step 1: Apply Spectral Correction files
fprintf('\nStep 1: Apply Spectral Correction \n')

%EEM correction factors
eemcormat = zeros(N_samples,N_em,N_ex);
for i = 2:N_samples+1
    eemcormat(i-1,:,:)=EmcorX(:,i)*ExcorX(:,i)'; %3D correction matrix
end

%correct the sample EEMs
Xc = eemcormat.*X; 
Xs=Xc; %spectrally corrected EEMs

%correct the Raman scans corresponding to the sample EEMs
ExcorlandaR = repmat(Excorlanda,size(EmcorR,1),1);
Rcormat=(ExcorlandaR.*EmcorR(:,2:end))';
Wc = [W(1,:); Rcormat.*W(2:end,:)];

%% Step 2: Inner Filter Correction (Parker & Barnes 1957)
%A is the absorbance in a 1 cm cell in decadal form (samples x wavelengths)
%method is accurate within 5 percent for A<1.5-2.0
%First, resize A to match wavelengths Ex and Em in EEMs

fprintf('Step 2: Apply Inner Filter correction \n')
A=CheckMatrixDim(A,N_samples+1,[],[],'A');
A=(sortrows(A',1))'; %wavelengths in descending order
if min(A)<0
    warning('MATLAB:NegAbs1','2.1: some Absorbance measurements have negative values.');
    warning('MATLAB:NegAbs2','2.2: Negative absorbance will be set to zero.');
    A(A<0)=0;
end

%Generate a matrix of inner filter corection factors
%according to the method of Parker & Barnes, 1957.
pathlength = 1; %1 cm cell
%warning('MATLAB:PathLen',['2.3: Pathlength of ' num2str(pathlength) '-cm is assumed.']);
IFCmat = IFEabs(A,N_samples,Em,Ex,pathlength);
% % OPTIONAL: Plot the correction factors
% for i=1:size(M,1)
%     figure, contourf(Ex,Em,squeeze(IFCmat(i,:,:)));
% end
Xife =Xc.*IFCmat;
Xc=Xife;

end

%% NESTED FUNCTIONS
function M=CheckMatrixDim(M,Dim1,Dim2,Dim3,Mname)
% M=CheckMatrixDim(M,Dim1,Dim2,Dim3,Mname)
% Check the dimensions of matrix M (name = 'Mname') against expected dimensions Dim1, Dim2,and Dim3
% Transpose M if necessary to achieve correct dimensions or else generate error message.
% Copyright K.R. Murphy, 
% July 2010
if isempty(Dim3); %2D data
    if size(M,1)~=Dim1;
        if and(size(M,2)==Dim1,size(M,1)==Dim2)
            M=M';
        elseif isempty(Dim2) %Number of columns is arbitrary
            if size(M,2)==Dim1
                M=M';
            else
                fprintf(['Check size of matrix ' Mname '.\n'])
                fprintf(['Expecting ' num2str(Dim1) ' rows.\n']),pause
                fprintf(['Current size is ' num2str(size(M,1)) ' rows and ' num2str(size(M,2)) ' columns.\n']),pause
                fprintf('Hit any key to continue.\n'),pause
                error('Unexpected Matrix Size')
            end
        else
            fprintf(['Check size of matrix ' Mname '.\n'])
            fprintf(['Expecting ' num2str(Dim1) ' rows and ' num2str(Dim2) ' columns.\n']),pause
            fprintf(['Current size is ' num2str(size(M,1)) ' rows and ' num2str(size(M,2)) ' columns.\n']),pause
            fprintf('Hit any key to continue.\n'),pause
            error('Unexpected Matrix Size')
        end
    end
else  %3D data
    if isequal(size(M),[Dim1 Dim2 Dim3]);
    else
        fprintf(['Check size of matrix ' Mname '.\n'])
        fprintf(['Expecting ' num2str(Dim1) ' x ' num2str(Dim2) ' x ' num2str(Dim3) '.\n']),pause
        fprintf(['Current size is ' num2str(size(M,1)) ' x ' num2str(size(M,2)) ' x ' num2str(size(M,3)) '.\n']),pause
        fprintf('Hit any key to continue.\n'),pause
        error('Unexpected Matrix Size')
    end
end
end

%% 
function Y=RepEEM(X,N,dochecks)
% Y=RepEEM(X,N,dochecks)
% Make an cube of identical EEMs, each consisting of the EEM X
% Size of cube specified by input N
% if dochecks = true, function displays 3 matrices which should all be identical.
% Copyright K.R. Murphy, 
% July 2010

n1=size(X,1);
n2=size(X,2);
X1 = repmat(X,N,1);
X1 = reshape(X1,[n1 N n2]);
Y = permute(X1,[2 1 3]);

if dochecks==true;
    squeeze(X(1:5,1:3))
    squeeze(Y(1,1:5,1:3))
    squeeze(Y(2,1:5,1:3))
    fprintf('check that the preceeding 3 matrices are all identical, then press any key to continue...')
    pause
end
end

%%
function Y=MatchWave(V1,V2,doplot)
%Y=MatchWave(V1,V2,doplot)
%Automatically match the size of two vectors, interpolating if necessary.
%Vector V1 is resized and interpolated to have the same wavelengths as V2;
%For example, V1 is a correction file (0.5nm intervals) and V2 is an Emission scan in 2 nm intervals.
%Errors are produced if 
%(1) there are wavelengths in V2 that are outside the upper or lower limit of V1.
%(2) Be aware of rounding errors. For example, 200.063 is NOT equivalent to 200 nm.
% Errors can often be resolved by restricting the wavelength range of V2
% Copyright K.R. Murphy
% July 2010

if size(V1,2)>2; V1=V1';end  %V1 should have multiple rows
if size(V2,2)>2; V2=V2';end  %V2 should have multiple rows

%Restrict the wavelength range of V1 so it is the same as V2
t=V1(find(V1(:,1)<=V2(1,1),1,'last'):find(V1(:,1)>=V2(end,1),1,'first'),:); 
if isempty(t)
    fprintf('\n')
    fprintf(['Error - Check that your VECTOR 1 (e.g. correction file) '...
        'encompasses \n the full range of wavelengths in VECTOR 2 (e.g. emission scan)\n'])
    fprintf('\n Hit any key to continue...')
    pause
    fprintf(['\n VECTOR 1 range is ' num2str(V1(1,1)) ' to ' num2str(V1(end,1)) ' in increments of ' num2str(V1(2,1)-V1(1,1)) '.']),pause
    fprintf(['\n VECTOR 2 range is ' num2str(V2(1,1)) ' to ' num2str(V2(end,1)) ' in increments of ' num2str(V2(2,1)-V2(1,1)) '.']),pause
    fprintf('\n\n This error can usually be resolved by restricting the wavelength range of VECTOR 2,');
    fprintf('\n also, be aware of rounding errors (e.g. 200.063 is NOT equivalent to 200 nm)\n');
    error('fdomcorrect012:wavelength','Error - abandoning calculations.');
else
    Y=[V2(:,1) interp1(t(:,1),t(:,2),V2(:,1))]; %V1 corresponding with EEM wavelengths
end

if doplot==true;    
figure, 
plot(V1(:,1),V1(:,2)), hold on, plot(Y(:,1),Y(:,2),'ro')
legend('VECTOR 1', 'VECTOR 2')
end
end

function M=IFEabs(A,N_samples,Em,Ex,PL)
%M = IFEabs(A,N_samples,Em,Ex,PL)
%Calculate a matrix of inner filter correction factors according to the
%absorbance method of Parker & Barnes (1957).
%Parker, C. A.; Barnes, W. J., Some experiments with spectrofluorimeters
%and filter fluorimeters. Analyst 1957, 82, 606-618.
%Inputs:
%A = 2D matrix of Absorbance scans with wavelength on the first row
%N_samples=number of samples
%Em=vector of emission wavelengths
%Ex=vector of excitation wavelengths
%PL = pathlength (1 cm cell)

N_em=length(Em);
N_ex=length(Ex);
M=NaN*ones([N_samples,N_em,N_ex]);

for k=1:N_samples;
    A254=A(2:end,round(A(1,:))==254);
    if max(A254>1.5)
        warning('MATLAB:InnerFilter',['2.4:  A254 exceeds 1.5 for one or more samples. ' ...
            'Confirm linearity between concentration and absorbance for A>1.5, or else ' ...
            'restrict the dataset to lower absorbance samples.']);
%         figure,
%         subplot(2,1,1),
%         plot(A(1,:)',A(2:end,:))
%         hold on, plot(A(1,:)',0.05*ones(size(A,2)),'k--')
%         hold on, plot(A(1,:)',1.5 *ones(size(A,2)),'k--')
%         hold on, plot(254*ones(11,1),0:0.1:1,'k--')
%         axis tight,
%         ylim([0 1])
%         xlabel('wavelength (nm)'); ylabel('Absorbance');
%         subplot(2,1,2),
%         plot(A254,'ro')
%         xlabel('sample number'); ylabel('Absorbance at 254 nm');
    end
    Atot=zeros(N_em,N_ex);
    Ak=[A(1,:);A(k+1,:)];
    
    %fprintf('matching Absorbance scans & Ex, Em \n');
    try
        Aem=MatchWave(Ak,Em,0);
    catch ME
        disp(ME)
        fprintf(['\nThe range of A is ' num2str(min(A(1,:))) '-' num2str(max(A(1,:)))]);
        fprintf(['\nThe range of Em is ' num2str(min(Em)) '-' num2str(max(Em))]);
        error('fdomcorrect012:InnerFilter:Wavelengths','2.41:  Absorbance scans must span the full range of Em');
    end
    
    try
        Aex=MatchWave(Ak,Ex,0);
    catch ME
        disp(ME)
        fprintf(['\nThe range of A is ' num2str(min(A(1,:))) '-' num2str(max(A(1,:)))]);
        fprintf(['\nThe range of Ex is ' num2str(min(Ex)) '-' num2str(max(Ex))]);
        error('fdomcorrect012:InnerFilter:Wavelengths','2.42:  Absorbance scans must span the full range of Ex');
    end
    
    for i=1:size(Aem,1)
        for j=1:size(Aex,1)
            Atot(i,j)=Aex(j,2)+Aem(i,2);
        end
    end
    IFC=10.^(PL/2*Atot);
    M(k,:,:)=IFC;
end
end

function newpair=nearestwave(inpair,Ex,Em)
%find wavelength pairs in Ex and Em as similar 
%to those listed in inpair as possible
newpair=inpair;
for i=1:size(inpair,1);
    [i1, j1]=min(abs(Ex-inpair(i,1))); %#ok<ASGLU>
    [i2, j2]=min(abs(Em-inpair(i,2))); %#ok<ASGLU>
    newpair(i,:)=[Ex(j1) Em(j2)];
end
end

function xlworkaround(FN,Sh)
% Work around for xlswrite, which does not work in the following situations:
%   1. The computer is a Mac
%   2. MS Excel is not installed
%   3. Communication between MATLAB and MS Excel fails for another reason.
% The work around is to use xlwrite instead of the built-in xlswrite.
%   Download ''XLWRITE'' (File ID: #38591 by Alec de Zegher) from the MATHWORKS 
%   file exchange, at http://www.mathworks.com/matlabcentral/fileexchange/');
%   Follow the instructions to install it, especially remembering to activate
%   the POI files according to the help documentation.
warning('MATLAB can not access MS Excel. Checking for alternative route via xlwrite.');
S = which('xlwrite');
if isempty(S)
    warning('xlwrite not on the MATLAB path. Observe the following instructions.');
    disp('1. Download ''XLWRITE'' (File ID: #38591 by Alec de Zegher) ')
    disp('   from the MATHWORKS file exchange, at');
    disp('      http://www.mathworks.com/matlabcentral/fileexchange/');
    disp('2. Unzip the folder and place it somewhere general ')
    disp('      e.g C:/Program Files/MATLAB/R2012b/extra_toolboxes/xlwrite')
    disp('3. Add the xlwrite path with subfolders to your MATLAB path. Save.')
    disp('4. Next, make the XLWRITE folder your current directory, then follow  ');
    disp('   the XLWRITE help (notes) to install the POI.');
    disp('5. Finally, change the current directory back to the location')
    disp('   where you want this file to be written, for example')
    disp('   >> cd ''C:/Data/EEMs/Project1/ ''  ');
    disp('6. Now try executing the code again.');
    disp('');
    error('MATLAB:No_MSExcel','Export to MS Excel spreadsheets not currently supported.')
else
    disp('Located xlwrite. Performing a quick test to see if it works.')
    try
        xlwrite(FN,{'xlwrite: Success!'},Sh,'A1');
        disp('Passed.')
    catch
        warning('Test failed. Will try adding POI library to the javapath...')
        currentdir=cd;
        cd(S);
        javaaddpath('poi_library/poi-3.8-20120326.jar');
        javaaddpath('poi_library/poi-ooxml-3.8-20120326.jar');
        javaaddpath('poi_library/poi-ooxml-schemas-3.8-20120326.jar');
        javaaddpath('poi_library/xmlbeans-2.3.0.jar');
        javaaddpath('poi_library/dom4j-1.6.1.jar');
        cd(currentdir)
        try
            xlwrite(FN,{'xlwrite: Success!'},Sh,'A1');
            disp('Passed.')
        catch %#ok<*CTCH>
            error('MATLAB:xlwrite','Writing to an Excel Spreadsheet using xlwrite work-around failed for unknown reasons')
        end
    end
end
end

function [Y,BaseArea]=RamanArea(M,EmMin,EmMax)
% [Y,EmMin,EmMax]=RamanAreaI(M,EmMin,EmMax)
% Find the area under the curves in M between wavelengths EmMin and EmMax, 
% data are first linearly interpolated to 0.5 nm intervals
% integration by the trapezoidal method
% murphyk@chalmers.se

%interpolate to 0.5 nm intervals
waveint=0.5; %nm
waves=(EmMin:waveint:EmMax)';
Mwaves=M(1,:)';
if or(EmMin<min(Mwaves),EmMax>max(Mwaves))
    disp(['Data range is ' num2str(min(Mwaves)) ' to ' num2str(max(Mwaves))])
    disp(['Integration range is ' num2str(min(waves)) ' to ' num2str(max(waves))])
    error('The chosen Raman integration range is outside the range of the measured data!')
end
Mpt5 = FastLinearInterp(Mwaves, M(2:end,:)', waves)'; %faster linear interp
%figure, plot(Mpt5)
%Mpt5 = interp1(M(1,:)', M(2:end,:)', waves,'spline')'; %built in alternative

%integrate
RAsum=trapz(waves,Mpt5,2);%;
BaseArea=(EmMax-EmMin)*(Mpt5(:,1)+0.5*(Mpt5(:,end)-Mpt5(:,1)));
%disp([RAsum BaseArea BaseArea./RAsum]);
Y = RAsum-BaseArea;
end

function Yi = FastLinearInterp(X, Y, Xi)
%by Jan Simon
% X and Xi are column vectros, Y a matrix with data along the columns
[dummy, Bin] = histc(Xi, X);  %#ok<ASGLU>
H            = diff(X);       % Original step size
% Extra treatment if last element is on the boundary:
sizeY = size(Y);
if Bin(length(Bin)) >= sizeY(1)
    Bin(length(Bin)) = sizeY(1) - 1;
end
Xj = Bin + (Xi - X(Bin)) ./ H(Bin);
% Yi = ScaleTime(Y, Xj);  % FASTER MEX CALL HERE
% return;
% Interpolation parameters:
Sj = Xj - floor(Xj);
Xj = floor(Xj);
% Shift frames on boundary:
edge     = (Xj == sizeY(1));
Xj(edge) = Xj(edge) - 1;
Sj(edge) = 1;           % Was: Sj(d) + 1;
% Now interpolate:
if sizeY(2) > 1
    Sj = Sj(:, ones(1, sizeY(2)));  % Expand Sj
end
Yi = Y(Xj, :) .* (1 - Sj) + Y(Xj + 1, :) .* Sj;
end

