function [ xls_range, csv_range, error ] = getrange( filename, varargin )
%GETRANGE reads a CSV file and returns both, CSV size and its equivalent excel XLS range.
%If the file has a header with NO numerical valid data the starting cell
%for the numerical_valid_range can be specified in VARARGIN{1} in the same 
%way as specified in XLS spreadsheet as [start_COL start_ROW end_COL end_ROW].
%It is also specify only the start range as [start_COL start_ROW]
%For the numerical_valid_start_range the starting cell is zero-based, then the 
%Excel cell 'A1' stands for [0 0].
%Example: the excel table starts with numerical range on excel 'A3' and ends in 'D8'
%Then numerical_valid_range = [0 2 3 7].
%
%This is useful to parametrize the 'readineems' drEEM  function
%This file is based on Matt G.'s 'XLSCOLNUM2STR' (authorid:28897) file
%(Matlab Central File Exchange)

% init
% como en CSVREAD el rango vlido se epecifica numricamente desde [0 0]
% (zero-based) es decir 'A1' --> [0 0] por eso 'A3' --> [0 2]
xls_range ='';
csv_range = [];
error = {};

% error checking
ind = strfind(filename,'.');
ext = filename(ind(end)+1:end);
switch ext
    case 'csv'
    otherwise
        error = {'ERROR getcsvrange:wrongFileFormat. File format NOT supported'};
        return;
end

switch nargin
    case 1
        csv_startC = 0;
        csv_startR = 0;
        xls_startC = {'A'};
        xls_startR = '1';
        data = csvread(filename,csv_startR,csv_startC); % data load
        csv_range = size(data);
        xls_endC = xlsColNum2Str(csv_range(2) + csv_startC);
        xls_endR = num2str(csv_startR + csv_range(1));
    case 2
        validrange = varargin{1}; % [startR startC endR endC]
        rows = size(validrange,2);
        switch rows
            case 2
                csv_startC = validrange(1);
                csv_startR = validrange(2);
                xls_startC = xlsColNum2Str(csv_startC+1);
                xls_startR = num2str(csv_startR+1);
                data = csvread(filename,csv_startR,csv_startC);
                csv_range = size(data);
                xls_endC = xlsColNum2Str(csv_range(2) + csv_startC);
                xls_endR = num2str(csv_startR + csv_range(1));
            case 4
                csv_startC = validrange(1);
                csv_startR = validrange(2);
                csv_endC = validrange(3);
                csv_endR = validrange(4);
                xls_startC = xlsColNum2Str(csv_startC+1);
                xls_startR = num2str(csv_startR+1);
                xls_endC = xlsColNum2Str(csv_endC+1);
                xls_endR = num2str(csv_endR+1);
            otherwise
                error = {'ERROR getcsvrange:wrongInputArguments. Too many input arguments in function'};
                return;
        end
    otherwise
        error = {'ERROR getcsvrange:wrongInputArguments. Too many input arguments in function'};
        return;
end
xls_range = [xls_startC{:} xls_startR '..' xls_endC{:} xls_endR];
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% sub-functions
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [colChar]=xlsColNum2Str(colNum)
%XLSCOLNUM2STR takes in an array of numbers and returns a cellular array
%of the same size with cell of corresponding Excel column names.
%
%For example:
%n=[1  10;
%   53 256]
%c=xlsColNum2Str(n);
%c={'A' , 'J';
%   'BA', 'IV'}
%Note: up to Excel 2003 the number of columns was limited to 256, as of
%Excel 2007 the number of columns has increased to 16,384 or 'XFD'
%This function is designed to take accept any integer so proper handling 
%of the number of columns should be taken care of outside this function

    colChar=cell(size(colNum)); %blank cell array
    
    % find max number of characters (AA n=2)
    numOfChars=ceil(max(colNum)/26)-1;
    n=1;
    
    while numOfChars>=1
        numOfChars=ceil(numOfChars/26)-1;
        n=n+1;
    end    
    
    remainder=num2cell(colNum);
    
    for s=n:-1:1
        if s>1
            %find limits
            % if n=2 then the columns go from AA to ZZ or 27 to 702
            L=sum(26.^(1:s-1))+1; % lower limit
            U=sum(26.^(1:s));   %upper limit
            %place current character to right of previous
            colChar(colNum>=L & colNum<=U) = ...
                cellfun(@(x,y) ([x char(ceil((y-(L-1))/26^(s-1))+64)]),...
                colChar(colNum>=L & colNum<=U),...      % x
                remainder(colNum>=L & colNum<=U),...    % y
                'UniformOutput',false);
            %calculate the remaining string
            %for example if last string was 'ABA' the 'A' was placed to the
            %right of the previous string and now 'BA' is remaining
            remainder(colNum>=L & colNum<=U)=...
                cellfun(@(x,y) (y-26^(s-1)*(double(x(end))-64)),...
                colChar(colNum>=L & colNum<=U),...
                remainder(colNum>=L & colNum<=U),'UniformOutput',false);
            colNum=cell2mat(remainder);
        else
             colChar=cellfun(@(x,y) ([x char(y+64)]),...
                 colChar,remainder,'UniformOutput',false);
        end
    end    
end