function [ abs, wls, units, error ] = hti2mat( filename )
%HTI2MAT imports absorbance values from a HITACHI spectrophotometer CSV 
%formatted file. Absorbance values (ABS) and wavelengths (WLS) are imported
%in ROWS
%
%	The script generates extra information that is not used but can be very
%	useful for future versions and an EEMs database implementation . This
%	information is labelled with (*)

% init
error = {};
abs = [];
wls = [];
units = 'AU'; % Absorbance Units (AU)
pflag = false; % plot flag, for debugging

% error checking
if ~exist(filename,'file')
    error = {'ERROR EEMlab:hti2mat:fileNotExist'};
    return;
end

% data load
fstring = fileread(filename);
fblocks = regexp(fstring, 'Data List', 'split');

% BLOCK 1 processing
% HEADER block. PARAMS: name, date, wl_start, end, step & speed
block = fblocks{1}; % block 1
lines = textscan(block,'%s','Delimiter','\r'); % read lines
line = textscan(lines{:}{3},'%s','Delimiter','\t');
absr_internal_name = line{:}{2}; % internal name given to the absorbance file(*)
line = textscan(lines{:}{5},'%s','Delimiter','\t');
date = line{:}{2}; % (*)
line = textscan(lines{:}{23},'%s','Delimiter','\t');
wl_end = str2double(line{:}{2}); % (*)
[~, rem] = strtok(line{:}{1},'(');
wl_unit = rem(2:3); % (*)
line = textscan(lines{:}{24},'%s','Delimiter','\t');
wl_start = str2double(line{:}{2}); % (*)
line = textscan(lines{:}{25},'%s','Delimiter','\t');
speed = str2double(line{:}{2}); % (*)
line = textscan(lines{:}{26},'%s','Delimiter','\t');
wl_step = str2double(line{:}{2}); % (*)
wl_length = 1+(wl_end-wl_start)/wl_step;

% BLOCK 2 processing
% HEADER block. PARAMS: name, date, wl_start, end, step & speed
block = fblocks{2}; % block 1
lines = textscan(block,'%s','Delimiter','\r'); % read lines
header = lines{:}(1:2);
info = lines{:}(3:3+wl_length-1); % isolate data for processing
absr = zeros(wl_length,2); % pre allocate eem for speed
for i = 1 : wl_length % we process line by line
    line = textscan(info{i},'%s','Delimiter','\t');
    absr(i,1) = str2double(line{:}{2});
    absr(i,2) = str2double(line{:}{3});
end
absr = sortrows(absr,1); % full absorbances range sorted in wl ascending order
absr(absr(:,2)<0,2) = 0; % correction of negative absorbances

% output values
ln = size(absr,1);
wls(1:ln) = absr(:,1); % wavelengths
abs(1:ln) = absr(:,2); % correspondent absorbance vaules

if (pflag)
    figure;
    plot(wls,abs);
    xlabel(['wavelength (' wl_unit ')']);
    ylabel(['absorbance (' units ')']);
    title('Absorbance');
    range = max(abs) - min(abs);
    axis([wl_start wl_end min(abs)-0.1*range max(abs)+0.1*range]);
    grid on;
end

end
