function [ eem_raw, ex, em, error ] = jscFL2mat( filename )
%JSC2MAT imports EEM raw data from a JASCO FP8300 TXT-formatted fluorometer file
%   A native JASCO FP8300 file is TXT formatted
%   The TXT reflects exactly what GXZ contains
%   'eem_raw' are raw intensity values
%
%   This script generates extra information that is not used but can be
%   very useful for future versions and an EEMs database implementation (
%   date, acquisition, sources...)

% init
error = {};
eem_raw = [];
ex = [];
em = [];

% checking file structure
if (isjscFL(filename,'JASCO') && isjscFL(filename,'FLUORESCENCE SPECTRUM'))
    
    % data load
    data = fileread(filename);
    
    % si se trata de un SCAN, eliminamos el bloque de 'extended data'
    % marcado con '#####'
    fblocks = regexp(data, '#####', 'split');
    data = fblocks{1};
    
    fblocks = regexp(data, 'XYDATA', 'split');
    % BLOCK 1 processing
    % HEADER block. PARAMS: acquisition date
    block = fblocks{1};
    lines = textscan(block,'%s','Delimiter','\r'); % read lines
    line = textscan(lines{:}{5},'%s','Delimiter','DATE'); % process 5th line (DATE)
    date = line{:}{end}; % this is the date_created in string
    line = textscan(lines{:}{6},'%s','Delimiter','TIME'); % process 6th line (TIME)
    time = line{:}{end}; % this is the date_created in string
    datetime = [date ' ' time]; % this is the date_created in string

    % BLOCK 2 processing
    % HEADER in lines 2 to 6. PARAMS: EEMs source, excitation & emission starts, ends and steps
    % EEM in lines 7 to 7+em_length-1. PARAMS: eem_raw & eem_cor
    % final </Group> line number 7+em_length. PARAMS: NONE
    block = fblocks{2};
    lines = textscan(block,'%s','Delimiter','\r'); % read lines
    
    % TRIMTEXTBLOCK(TEXT) --> fa un trim del bloc de text passat com a
    % paràmetre, eliminant les files buides de l'inici i del final del bloc
    firstLine = lines{:}(1);
    while (isempty(firstLine{:}))
        lines{:}(1) = []; % delete first EMPTY line
        firstLine = lines{:}(1);        
    end
    lastLine = lines{:}(end);
    while (isempty(lastLine{:}))
        lines{:}(end) = []; % delete last EMPTY line
        lastLine = lines{:}(end);
    end

    ex = textscan(lines{:}{1},'%u','Delimiter',' ');
    ex = ex{:}'; % excitation wavelengths in the 2nd row
    lines{:}(1) = []; % delete 1st ALREADY USED line with excitation wavelengths
    ex_length = length(ex);
    em_length = length(lines{:});
    eem_raw = zeros(em_length, ex_length);
    em = zeros(1, em_length);
    for i=1:length(lines{:})
        line = textscan(lines{:}{i},'%f','Delimiter',' ');
        em(i) = line{:}(1);
        eem_raw(i,:) = line{:}(2:end)';
    end
    em = uint32(em);

    % error checking
    if (size(eem_raw,1) ~= length(em)) || ...
            (size(eem_raw,2) ~= length(ex))
        % the size of the EEM does not match with excitation and/or emission wavelengths arrays
        error{end+1} = 'ERROR jscFL2mat:emmSizeMismatch';
        return;
    end
    if  ~isempty(find(diff(diff(ex)) ~= 0, 1 ))
        % some values are missed from excitation wavelengths array
        error{end+1} = 'ERROR jscFL2mat:excitationArrayMissedValues';
        return;
    end
    if  ~isempty(find(diff(diff(em)) ~= 0, 1 ))
        % some values are missed from emission wavelengths array
        error{end+1} = 'ERROR jscFL2mat:emissionArrayMissedValues';
        return;
    end
else
    % the QuantaMaster file does not include one <Session> and two <Group> elements
    error{end+1} = 'ERROR jscFL2mat:jascoFileFormatWrong';
    return;
end

end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% sub-functions

function [ flag ] = isjscFL( filename, tag )
%ISJSC is TRUE if the file is JASCO FP8300 formatted and FALSE otherwise

flag = false;
ind = strfind(filename,'.'); % we get file extension
format = filename(ind(end)+1:end);
if ~strcmpi(format,'TXT')
    return;
else
    fstring = fileread(filename);
    flag = ~isempty(regexp(fstring, tag, 'once'));
end
end
