function [ eem_raw, eem_COR, eem_Corr, ex, em, error ] = qms2mat( filename )
%QMS2MAT imports EEM raw data from a QUANTAMASTER fluorometer file
%	Raw data are drEEM toolbox formatted (data)
%   A native QuantaMaster file is GXZ formatted and saved as TXT
%   The TXT reflects exactly what GXZ contains
%   'eem_raw' are raw intensity values
%   'eem_COR' are corrected intensity values
%   'eem_Corr' are Ex/Em spectral corrections automatically applied to intensity values
%   for more information please read the Quantamaster datasheet
%
%   This script generates extra information that is not used but can be
%   very useful for future versions and an EEMs database implementation (
%   date, acquisition, sources...)

% init
error = {};
eem_raw = [];
eem_COR = [];
eem_Corr = [];
ex = [];
em = [];

% checking file structure
if (isqms(filename,'Session',1) && isqms(filename,'Group',2))
    
    % data load
    data = fileread(filename);
    fblocks = regexp(data, '<Group>', 'split');
    
    % BLOCK 1 processing
    % HEADER block. PARAMS: date & acquisition
    block = fblocks{1};
    lines = textscan(block,'%s','Delimiter','\r'); % read lines
    line = textscan(lines{:}{2},'%s','Delimiter',' '); % process 2nd line
    datetime = [line{:}{end-1} ' ' line{:}{end}]; % this is the date_created in string
    acquisition = [];
    for i = 1 : length(line{:}) - 2
        acquisition = [acquisition line{:}{i} ' '];
    end
    acquisition = deblank(acquisition); % remove the last added whitespace

    % BLOCK 2 processing
    % HEADER in lines 2 to 6. PARAMS: EEMs source, excitation & emission starts, ends and steps
    % EEM in lines 7 to 7+em_length-1. PARAMS: eem_raw & eem_cor
    % final </Group> line number 7+em_length. PARAMS: NONE
    block = fblocks{2};
    lines = textscan(block,'%s','Delimiter','\r'); % read lines
    header = lines{:}(2:6); % header processing
    eem_raw_source = header{1};
    eem_COR_source = header{1};
    ex_length = str2double(header{2})/2; % excitation length must be integer
    if (rem(ex_length,1) > 0)
        % excitation length in raw and COR EEMs must be an integer
        error{end+1} = 'ERROR qms2mat:excitationArrayLengthWrong';
        return;
    end
    line = textscan(header{3},'%s','Delimiter','\t');
    em_length = str2double(line{:}{1}); % emission length
    line = textscan(header{4},'%s','Delimiter','\t'); % line 5
    [ex1,em1] = strtok(line{:}{1},':');
    ex_start = str2double(ex1(end-2:end));
    em_start = str2double(em1(2:4));
    em_end = str2double(em1(end-2:end));
    em_step = (em_end - em_start)/(em_length-1);
    [ex2,em2] = strtok(line{:}{2*ex_length-1},':'); % last excitation wavelength
    ex_end = str2double(ex2(end-2:end));
    ex_step = (ex_end - ex_start)/(ex_length-1);
    if isnan(ex_step)
        ex = ex_start; % just ONE excitation wavelength
    else
        ex = ex_start:ex_step:ex_end; % normal range of excitation wls
    end
    em = em_start:em_step:em_end;
    % Block 2 EEM data processing
    data = lines{:}(7:7+em_length-1); % isolate data for processing
    data = strrep(data,',','.'); % we change ',' by '.' for numeric conversion
    eem_raw = zeros(em_length,ex_length); % pre allocate eem for speed
    eem_COR = zeros(em_length,ex_length); % pre allocate eem for speed
    if (length(data) == em_length)
        for i = 1 : length(data) % we process line by line
            line = textscan(data{i},'%s','Delimiter','\t');
            k = 1;
            for j = 2 : 2: 2*ex_length
                eem_raw(i,k) = str2double(line{:}{j});
                eem_COR(i,k) = str2double(line{:}{2*ex_length+j});
                k = k + 1;
            end
        end
        % Possible solution --> eem_COR(eem_COR < 0) = NaN; % correct  negative values due to bad fluorometer calibration
    else
        % emission length in raw and COR EEMs does not match with data length
        error{end+1} = 'ERROR qms2mat:inconsistentData';
        return;
    end

    % BLOCK 3 processing
    % HEADER in lines 2 to 6. PARAMS: EEM source
    % final </Group> line number 7+em_length. PARAMS: NONE
    block = fblocks{3};
    lines = textscan(block,'%s','Delimiter','\r'); % read lines
    header = lines{:}(2:6); % header processing
    eem_Corr_source = header{1};
    if (str2double(header{2}) ~= ex_length)
        % excitation length in Corr EEM does not match with raw and COR EEMs
        error{end+1} = 'ERROR qms2mat:excitationArrayLengthWrong';
        return;
    end
    % Block 3 EEM data processing
    data = lines{:}(7:7+em_length-1); % isolate data for processing
    data = strrep(data,',','.'); % we change ',' by '.' for numeric conversion
    eem_Corr = zeros(em_length,ex_length); % pre allocate eem for speed
    if (length(data) == em_length)
        for i = 1 : length(data) % we process line by line
            line = textscan(data{i},'%s','Delimiter','\t');
            k = 1;
            for j = 2 : 2: 2*ex_length
                eem_Corr(i,k) = str2double(line{:}{j});
                k = k + 1;
            end
        end
    else
        % emission length in Corr EEM does not match with data length
        error{end+1} = 'ERROR qms2mat:inconsistentData';
        return;
    end
    
    % error checking
    if ~isempty(setdiff(size(eem_raw),size(eem_COR))) || ...
            ~isempty(setdiff(size(eem_COR),size(eem_Corr))) || ...
            ~isempty(setdiff(size(eem_raw),size(eem_Corr)))
        % the size of the EEMs is not consistent between them
        error{end+1} = 'ERROR qms2mat:emmSizeMismatch';
        return;
    end
    if (size(eem_raw,1) ~= length(em)) || ...
            (size(eem_raw,2) ~= length(ex))
        % the size of the EEM does not match with excitation and/or emission wavelengths arrays
        error{end+1} = 'ERROR qms2mat:emmSizeMismatch';
        return;
    end
    if  ~isempty(find(diff(diff(ex)) ~= 0, 1 ))
        % some values are missed from excitation wavelengths array
        error{end+1} = 'ERROR qms2mat:excitationArrayMissedValues';
        return;
    end
    if  ~isempty(find(diff(diff(em)) ~= 0, 1 ))
        % some values are missed from emission wavelengths array
        error{end+1} = 'ERROR qms2mat:emissionArrayMissedValues';
        return;
    end
else
    % the QuantaMaster file does not include one <Session> and two <Group> elements
    error{end+1} = 'ERROR qms2mat:quantamasterFileFormatWrong';
    return;
end

end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% sub-functions

function [ flag ] = isqms( filename, tag, nots )
%ISQMS is TRUE if the file is QUANTAMASTER formatted and FALSE otherwise
% NOTS is the Number Of TagS of that kind that appear in the EEM file

flag = false;
ind = strfind(filename,'.'); % we get file extension
format = filename(ind(end)+1:end);
if ~strcmpi(format,'TXT')
    return;
else
    fstring = fileread(filename);
    tag_start = regexp(fstring, ['<' tag '>']);
    tag_end = regexp(fstring, ['</' tag '>']);
    flag = ((length(tag_start) == nots) && ...
        (length(tag_end) == nots) && ...
        all(tag_start < tag_end));
end
end
