function [ A, W, wave, filelist, error ] = readinscans4hti( varargin )
%READINSCANS4HTI reads in all CSV formatted HITACHI spectrophotometer scan
%files in the corrent folder returning values into a 2D matrix. Please
%remove non-scan files of specified format from the current directory
%before running the script. The files will be loaded in alphabetical order
%(see filelist). Input scans must be of the same size, type and format. For
%a correct error checking the first file in folder must be a valid scan
%file.
%
%   INTERNAL FUNCTIONS:
%   QMS2MAT, GETFILESINFOLDER
%
%   USAGE:
%
%   INPUTS:
%   If there are no inputs the function processes all the files in the
%   current directory. If we pass one input, varargin{1} is the concrete
%   list of files (cell array) to be processed

% PARSING INPUTS
if isempty(varargin)
    rflag = false; % recursive flag
    pflag = false; % prepend flag
    files = getfilesinfolder('.',rflag,pflag);
else
    files = varargin{1};
end
if ~iscell(files) % just a file name passed as string argument
    files = cellstr(files);
end
nfiles = length(files);

% INIT
A = [];
W = [];
wave = [];
filelist = cell(nfiles,1);
error = cell(nfiles,1);
ok = nan(1,nfiles); % index for valid scans

%getting values for the first time to preallocate variables for speed
%we suppose that the first file is a VALID file. Otherwise the load will
%fail for all the files in folder
[~,wls_preallocate,~,err] = hti2mat(files{1});
if isempty (err)
    wlmin = min(wls_preallocate);
    wlmax = max(wls_preallocate);
else % the first file is wrong
    error = {'ERROR readinscans4hti:noValidAbsorbanceFiles'};
    return;
end
for i = 2 : nfiles
    [~,wls,~,err] = hti2mat(files{i});
    if isempty (err)
        % for the later interpolation, we ensure that all the scans will
        % have values in the wavelength interval
        wlmin = max(wlmin,wls(1));
        wlmax = min(wlmax,wls(end));
    end
end
scanwl = wlmin:wlmax;
wlsn = length(scanwl); % number of wavelengths included in the scan
scan = [scanwl; zeros(1,wlsn)];
A = nan(nfiles,wlsn); % preallocate variable for speed
W = nan(nfiles,wlsn);
wave = nan(1,wlsn);
error = cell(nfiles,1); % reset the error log

% start iterations for every file in directory
for i = 1 : nfiles
    disp(files{i});
    [abs,wls,~,err] = hti2mat(files{i});
    if isempty(err)
        mabs = MatchWave([wls;abs],scan,false); % matched absorbance
        A(i,:) = mabs(:,2)';
        W(i,:) = mabs(:,1)';
        filelist(i) = files(i);
        ok(i) = 1;
    else
        error{i} = [err{:} '. Scan ''' files{i} ''' not loaded in the dataset'];
        return;
    end
end

% remove empty cells
A = A(~isnan(ok),:);
W = W(~isnan(ok),:);
filelist = filelist(~isnan(ok),:);
filelist = char(filelist{:}); % transform cell array 'filelist' into a matrix of strings (as in drEEM)
error = error(isnan(ok));

%HOW TO GET THE 'WAVE' VARIABLE. CODE TAKEN BY READINSCANS (DREEM TOOLBOX)
%Check for consistency among wavelengths
rndW = round(W*2)/2;
rndW1 = repmat(rndW(1,:),[size(rndW,1) 1]);
Wdiff = rndW-rndW1;
if max(max(Wdiff)) == 0
    wave = rndW(1,:);
else
    fprintf('\n')
    fprintf('Warning: not all input wavelengths were the same! \n')
    fprintf('Check the rows of output variable W for non-standard wavelengths.\n')
    fprintf('The output variable "wave" is a vector of NaNs.\n')
end

end

%% ===== SUB-FUNCTIONS from drEEM
function Y=MatchWave(V1,V2,doplot)
%Y=MatchWave(V1,V2,doplot)
%Automatically match the size of two vectors, interpolating if necessary.
%Vector V1 is resized and interpolated to have the same wavelengths as V2;
%For example, V1 is a correction file (0.5nm intervals) and V2 is an Emission scan in 2 nm intervals.
%Errors are produced if 
%(1) there are wavelengths in V2 that are outside the upper or lower limit of V1.
%(2) Be aware of rounding errors. For example, 200.063 is NOT equivalent to 200 nm.
% Errors can often be resolved by restricting the wavelength range of V2
% Copyright K.R. Murphy
% July 2010

if size(V1,2)>2; V1=V1';end  %V1 should have multiple rows
if size(V2,2)>2; V2=V2';end  %V2 should have multiple rows

%Restrict the wavelength range of V1 so it is the same as V2
t=V1(find(V1(:,1)<=V2(1,1),1,'last'):find(V1(:,1)>=V2(end,1),1,'first'),:); 
if isempty(t)
    fprintf('\n')
    fprintf(['Error - Check that your VECTOR 1 (e.g. correction file) '...
        'encompasses \n the full range of wavelengths in VECTOR 2 (e.g. emission scan)\n'])
    fprintf('\n Hit any key to continue...')
    pause
    fprintf(['\n VECTOR 1 range is ' num2str(V1(1,1)) ' to ' num2str(V1(end,1)) ' in increments of ' num2str(V1(2,1)-V1(1,1)) '.']),pause
    fprintf(['\n VECTOR 2 range is ' num2str(V2(1,1)) ' to ' num2str(V2(end,1)) ' in increments of ' num2str(V2(2,1)-V2(1,1)) '.']),pause
    fprintf('\n\n This error can usually be resolved by restricting the wavelength range of VECTOR 2,');
    fprintf('\n also, be aware of rounding errors (e.g. 200.063 is NOT equivalent to 200 nm)\n');
    error('fdomcorrect:wavelength','Error - abandoning calculations.');
else
    Y=[V2(:,1) interp1(t(:,1),t(:,2),V2(:,1))]; %V1 corresponding with EEM wavelengths
end

if doplot==true;    
figure, 
plot(V1(:,1),V1(:,2)), hold on, plot(Y(:,1),Y(:,2),'ro')
legend('VECTOR 1', 'VECTOR 2')
end
end
