function [ S, W, wave, filelist, error ] = readinscans4jsc( varargin )
%READINSCANS4JSC reads in a list of JASCO FP8300 TXT-formatted fluorometer
%SCAN filess getting a 3D matrix of EEMs. Please remove non-EEM files from the
%current directory before running the script. The files will be loaded in
%alphabetical order (see output file list). All the input EEMs must be of
%the same size, type, format and in the same Ex, Em wavelength range.
%
%   INTERNAL FUNCTIONS:
%   JSC2EEM, GETFILESINFOLDER
%
%   USAGE:
%
%   INPUTS:
%   If there are no inputs the function processes all the files in the
%   current directory. If we pass one input, varargin{1} is the concrete
%   list of files (cell array) to be processed

% PARSING INPUTS
if isempty(varargin)
    rflag = false; % recursive flag
    pflag = false; % prepend flag
    filelist = getfilesinfolder('.',rflag,pflag);
else
    filelist = varargin{1};
end
if ~iscell(filelist) % just a file name passed as string argument
    filelist = cellstr(filelist);
end

% INIT
nfiles = length(filelist);
S = [];
W = [];
wave = [];
error = cell(nfiles,1);
ok = nan(1,nfiles); % index for valid scans
preallocated = false;

% start iterations for every file in directory
for i = 1 : nfiles
    disp(filelist{i});
    % checking file structure
    if (isjsc(filelist{i},'JASCO') && isjsc(filelist{i},'FLUORESCENCE SPECTRUM'))
        
        % data load
        data = fileread(filelist{i});
        
        % si se trata de un SCAN, eliminamos el bloque de 'extended data'
        % marcado con '#####'
        fblocks = regexp(data, '#####', 'split');
        data = fblocks{1};
        
        fblocks = regexp(data, 'XYDATA', 'split');
        % BLOCK 1 processing
        % HEADER block. PARAMS: acquisition date
        %         block = fblocks{1};
        %         lines = textscan(block,'%s','Delimiter','\r'); % read lines
        %         line = textscan(lines{:}{5},'%s','Delimiter','DATE'); % process 5th line (DATE)
        %         date = line{:}{end}; % this is the date_created in string
        %         line = textscan(lines{:}{6},'%s','Delimiter','TIME'); % process 6th line (TIME)
        %         time = line{:}{end}; % this is the date_created in string
        %         datetime = [date ' ' time]; % this is the date_created in string
        
        % BLOCK 2 processing
        % HEADER in lines 2 to 6. PARAMS: EEMs source, excitation & emission starts, ends and steps
        % EEM in lines 7 to 7+em_length-1. PARAMS: eem_raw & eem_cor
        % final </Group> line number 7+em_length. PARAMS: NONE
        block = fblocks{2};
        lines = textscan(block,'%s','Delimiter','\r'); % read lines
        
        % TRIMTEXTBLOCK(TEXT) --> fa un trim del bloc de text passat com a
        % paràmetre, eliminant les files buides de l'inici i del final del bloc
        firstLine = lines{:}(1);
        while (isempty(firstLine{:}))
            lines{:}(1) = []; % delete first EMPTY line
            firstLine = lines{:}(1);
        end
        lastLine = lines{:}(end);
        while (isempty(lastLine{:}))
            lines{:}(end) = []; % delete last EMPTY line
            lastLine = lines{:}(end);
        end
        % at this point we have just CLEAN data (wavelengths and fluorescence)
        
        if ~preallocated % just for the FIRST file, to preallocate variables for speed
            preallocated = true;
            wlsn = length(lines{:}); % number of wavelengths included in the scan
            S = nan(nfiles,wlsn); % preallocate variable for speed
            W = nan(nfiles,wlsn);
            wave = nan(1,wlsn);
            error = cell(nfiles,1); % reset the error log
        end
        if (length(lines{:}) ~= wlsn)
            W(i,:) = nan(1,wlsn);
            S(i,:) = nan(1,wlsn);
            error{i} = ['ERROR readinscans4jsc:wrongScanLength. The length of the scan ', ...
                filelist{i} ' does not match with the length of the dataset'];
        else
            em = zeros(1,wlsn);
            scan = zeros(1,wlsn);
            for j=1:length(lines{:})
                line = textscan(lines{:}{j},'%f','Delimiter',' ');
                em(j) = line{:}(1);
                scan(j) = line{:}(2);
            end
            ok(i) = 1; % valid scan
            W(i,:) = uint32(em);
            S(i,:) = scan;
        end
    end
end

% remove NON-VALID cells
S = S(~isnan(ok),:);
W = W(~isnan(ok),:);
filelist = filelist(~isnan(ok));
filelist = char(filelist{:}); % transform cell array 'filelist' into a matrix of strings (as in drEEM)
error = error(isnan(ok));

%HOW TO GET THE 'WAVE' VARIABLE. CODE TAKEN BY READINSCANS (DREEM TOOLBOX)
%Check for consistency among wavelengths
rndW = round(W*2)/2;
rndW1 = repmat(rndW(1,:),[size(rndW,1) 1]);
Wdiff = rndW-rndW1;
if max(max(Wdiff)) == 0
    wave = rndW(1,:);
else
    fprintf('\n')
    fprintf('Warning: not all input wavelengths were the same! \n')
    fprintf('Check the rows of output variable W for non-standard wavelengths.\n')
    fprintf('The output variable "wave" is a vector of NaNs.\n')
end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% sub-functions

function [ flag ] = isjsc( filename, tag )
%ISJSC is TRUE if the file is JASCO FP8300 formatted and FALSE otherwise

flag = false;
ind = strfind(filename,'.'); % we get file extension
format = filename(ind(end)+1:end);
if ~strcmpi(format,'TXT')
    return;
else
    fstring = fileread(filename);
    flag = ~isempty(regexp(fstring, tag, 'once'));
end
end
