function [ S, W, wave, filelist, error ] = readinscans4qms( varargin )
%READINSCANS4QMS reads in all TXT formatted QuantaMaster fluorometre SCAN
%files in the corrent folder returning values into a 2D matrix. Please
%remove non-scan files of specified format from the current directory
%before running the script. The files will be loaded in alphabetical order
%(see filelist). Input scans must be of the same size, type and format with
%the same emission wavelengths and exctited to just one wavelength 
%(length(ex) == 1). For a correct error checking the first file in folder
%must be a valid scan file.
%
%   INTERNAL FUNCTIONS:
%   QMS2MAT, GETFILESINFOLDER
%
%   USAGE:
%
%   INPUTS:
%   If there are no inputs the function processes all the files in the
%   current directory. If we pass one input, varargin{1} is the concrete
%   list of files (cell array) to be processed

% PARSING INPUTS
if isempty(varargin)
    rflag = false; % recursive flag
    pflag = false; % prepend flag
    files = getfilesinfolder('.',rflag,pflag);
else
    files = varargin{1};
end
if ~iscell(files) % just a file name passed as string argument
    files = cellstr(files);
end
nfiles = length(files);

% INIT
S = [];
W = [];
wave = [];
filelist = cell(nfiles,1);
error = cell(nfiles,1);
ok = nan(1,nfiles); % index for valid scans

%getting values for the first time to preallocate variables for speed
%we suppose that the first file is a VALID file. Otherwise the load will
%fail for all the files in folder
i = 1;
err = {'not empty'};
while ~isempty(err)
    [~,~,~,~,em_preallocate,err] = qms2mat(files{i});
    error{i} = [err{:} '. File ''' files{i} ''' NOT loaded in the dataset'];
    i = i + 1;
    if i > nfiles+1 % NONE of the files in directory is valid
        return;
    end
end
wlsn = length(em_preallocate); % number of wavelengths included in the scan
S = nan(nfiles,wlsn); % preallocate variable for speed
W = nan(nfiles,wlsn);
wave = nan(1,wlsn);
error = cell(nfiles,1); % reset the error log

% start iterations for every file in directory
for i = 1 : nfiles
    disp(files{i});
    [~,eem_COR,~,ex,em,err] = qms2mat(files{i});
    if isempty(err)
        if length(ex) == 1 % this a a scan (array of values)
            if length(em) == wlsn
                S(i,:) = eem_COR';
                W(i,:) = em;
                filelist(i) = files(i);
                ok(i) = 1;
            else
                error{i} = ['ERROR readinscans4qms:wrongScanLength. The length of the scan ', ...
                    files{i} ' does not match with the length of the dataset'];
            end
        else % this is not a scan but a EEM (values in a matrix)
            error{i} = ['ERROR readinscans4qms:wrongExcitationLength. The file ''', ...
                files{i} ''' is not a scan as it is excited to several wavelengths.', ...
                ' Try to use ''readineems4qms'' instead'];
        end
    else
        error{i} = [err{:} '. Scan ''' files{i} ''' not loaded in the dataset'];
    end
end

% remove empty cells
S = S(~isnan(ok),:);
W = W(~isnan(ok),:);
filelist = filelist(~isnan(ok),:);
filelist = char(filelist{:}); % transform cell array 'filelist' into a matrix of strings (as in drEEM)
error = error(isnan(ok));

%HOW TO GET THE 'WAVE' VARIABLE. CODE TAKEN BY READINSCANS (DREEM TOOLBOX)
%Check for consistency among wavelengths
rndW = round(W*2)/2;
rndW1 = repmat(rndW(1,:),[size(rndW,1) 1]);
Wdiff = rndW-rndW1;
if max(max(Wdiff)) == 0
    wave = rndW(1,:);
else
    fprintf('\n')
    fprintf('Warning: not all input wavelengths were the same! \n')
    fprintf('Check the rows of output variable W for non-standard wavelengths.\n')
    fprintf('The output variable "wave" is a vector of NaNs.\n')
end

end